//
//  Utils.m
//  MoveAdvisor
//
//  Created by Outsourcing IT Services Ltd. on 5/27/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "AppDelegate.h"
#import "Utils.h"
#import "Text.h"

@implementation Utils

+ (NSManagedObjectContext *)getContext {
    return ((AppDelegate *)[UIApplication sharedApplication].delegate).managedObjectContext;
}

+ (void)saveContext {
    [(AppDelegate *)([UIApplication sharedApplication]).delegate saveContext];
}

+ (UIColor *)getColor:(NSUInteger)color {
    CGFloat alpha = (color >> 24) & 0xff;
    CGFloat red = (color >> 16) & 0xff;
    CGFloat green = (color >> 8) & 0xff;
    CGFloat blue = color & 0xff;
    return [UIColor colorWithRed:red / 255 green:green / 255 blue:blue / 255 alpha:alpha / 255];
}

+ (NSString *)getString:(NSDictionary *)data key:(NSString *)key {
    id obj = [data objectForKey:key];
    if (obj != nil && [obj isKindOfClass:[NSString class]]) {
        return (NSString *)obj;
    } else {
        return nil;
    }
}

+ (NSNumber *)getNumber:(NSDictionary *)data key:(NSString *)key {
    id obj = [data objectForKey:key];
    if (obj != nil && [obj isKindOfClass:[NSNumber class]]) {
        return (NSNumber *)obj;
    } else {
        return nil;
    }
}

+ (NSArray *)getArray:(NSDictionary *)data key:(NSString *)key {
    id obj = [data objectForKey:key];
    if (obj != nil && [obj isKindOfClass:[NSArray class]]) {
        return (NSArray *)obj;
    } else {
        return nil;
    }
}

+ (NSDictionary *)getDictionary:(NSDictionary *)data key:(NSString *)key {
    id obj = [data objectForKey:key];
    if (obj != nil && [obj isKindOfClass:[NSDictionary class]]) {
        return (NSDictionary *)obj;
    } else {
        return nil;
    }
}

+ (UIImage *)loadImage:(NSString *)url download:(void (^)(UIImage *image))download {
    NSError *error;
    NSURL *file = [[NSFileManager defaultManager] URLForDirectory:NSDocumentDirectory inDomain:NSUserDomainMask appropriateForURL:nil create:NO error:&error];
    if(url != nil) {
        file = [file URLByAppendingPathComponent:@"images"];
        if(![[NSFileManager defaultManager] fileExistsAtPath:file.path]) {
            [[NSFileManager defaultManager] createDirectoryAtPath:file.path withIntermediateDirectories:NO attributes:nil error:&error];
        }
        if(url != nil && ![url isEqualToString:@""]) {
            int pos = (int)url.length - 1;
            while(pos >= 0 && [url characterAtIndex:pos] != '/') {
                pos--;
            }
            file = [file URLByAppendingPathComponent:[url substringFromIndex:pos + 1]];
            int size = 0;
            NSInteger i;
            NSMutableArray *images = ((AppDelegate *)[UIApplication sharedApplication].delegate).images;
            NSInteger current = -1;
            for(i = images.count - 1; i >= 0; i--) {
                if(size < 10 * 1024 * 1024) {
                    if([(NSString *)[(NSDictionary *)[images objectAtIndex:i] objectForKey:@"name"] isEqualToString:[file lastPathComponent]]) {
                        current = i;
                    }
                    size += ((NSNumber *)[(NSDictionary *)[images objectAtIndex:i] objectForKey:@"size"]).intValue;
                }
                else {
                    break;
                }
            }
            if(current >= 0) {
                NSDictionary *dictionary = (NSDictionary *)[images objectAtIndex:current];
                [images removeObjectAtIndex:current];
                [images addObject:dictionary];
            }
            while(i >= 0) {
                NSString *file = (NSString *)[(NSDictionary *)[images objectAtIndex:i] objectForKey:@"file"];
                if(file != nil) {
                    [[NSFileManager defaultManager] removeItemAtPath:file error:nil];
                    [images removeObjectAtIndex:i];
                }
                i--;
            }
            if(![[NSFileManager defaultManager] fileExistsAtPath:file.path]) {
                NSURLSessionConfiguration *configuration = [NSURLSessionConfiguration defaultSessionConfiguration];
                NSURLSession *session = [NSURLSession sessionWithConfiguration:configuration delegate:nil delegateQueue:[NSOperationQueue mainQueue]];
                NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:url]];
                NSURLSessionDownloadTask *task = [session downloadTaskWithRequest:request completionHandler:^(NSURL *location, NSURLResponse *response, NSError *error) {
                    if(location != nil) {
                        NSError *fileerror;
                        if([[NSFileManager defaultManager] moveItemAtURL:location toURL:file error:&fileerror]) {
                            NSDictionary *fileattributes = [[NSFileManager defaultManager] attributesOfItemAtPath:[file path] error:nil];
                            NSDictionary *dictionary = [NSDictionary dictionaryWithObjectsAndKeys:
                                                        [fileattributes objectForKey:NSFileSize], @"size",
                                                        [file lastPathComponent], @"name",
                                                        [file path], @"file",
                                                        nil];
                            [images addObject:dictionary];
                            UIImage *image = [UIImage imageWithContentsOfFile:file.path];
                            if(image != nil) {
                                download(image);
                            }
                        }
                    }
                }];
                [task resume];
                return nil;
            }
            else {
                return [UIImage imageWithContentsOfFile:file.path];
            }
        }
        else {
            return nil;
        }
    }
    else {
        return nil;
    }
}

+ (BOOL)validateName:(NSString *)name {
    NSRange range = [name rangeOfString:@" "];
    if(range.location != NSNotFound) {
        NSString *first = [[name substringToIndex:range.location] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
        NSString *last = [[name substringFromIndex:range.location] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
        return ![first isEqualToString:@""] && ![last isEqualToString:@""];
    }
    else {
        return NO;
    }
}

+ (BOOL)validatePhone:(NSString *)phone {
    if(phone.length == 12) {
        if([[phone substringToIndex:2] isEqualToString:@"+1"]) {
            if([[[phone substringFromIndex:2] stringByTrimmingCharactersInSet:[NSCharacterSet decimalDigitCharacterSet]] isEqualToString:@""]) {
                return YES;
            }
            else {
                return NO;
            }
        }
        else {
            return NO;
        }
    }
    else if(phone.length == 10) {
        if([[[phone substringFromIndex:2] stringByTrimmingCharactersInSet:[NSCharacterSet decimalDigitCharacterSet]] isEqualToString:@""]) {
            return YES;
        }
        else {
            return NO;
        }
    }
    else {
        return NO;
    }
}

+ (BOOL)validateEmail:(NSString *)email {
    NSError *error;
    NSRegularExpression *regex = [NSRegularExpression regularExpressionWithPattern:@"^[\\w!#$%&'*+/=?`{|}~^-]+(?:\\.[\\w!#$%&'*+/=?`{|}~^-]+)*@(?:[a-zA-Z0-9-]+\\.)+[a-zA-Z]{2,6}$" options:NSRegularExpressionCaseInsensitive error:&error];
    if(regex != nil) {
        return [regex numberOfMatchesInString:email options:0 range:NSMakeRange(0, email.length)] > 0;
    }
    else {
        return NO;
    }
}

+ (BOOL)validateEmails:(NSString *)content {
    NSArray<NSString *> *emails = [content componentsSeparatedByCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
    for(NSString *email in emails) {
        if(![Utils validateEmail:email]) {
            return NO;
        }
    }
    return YES;
}

+ (NSString *)getText:(NSString *)text {
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Text"];
    request.predicate = [NSPredicate predicateWithFormat:@"name = %@", text];
    NSManagedObjectContext *context = [Utils getContext];
    NSError *error;
    NSArray *texts = [context executeFetchRequest:request error:&error];
    if(texts != nil && texts.count > 0) {
        NSString *text = ((Text *)[texts objectAtIndex:0]).value;
        return text;
    }
    else {
        return nil;
    }
}

+ (void)setText:(UIView *)control text:(NSString *)text {
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Text"];
    request.predicate = [NSPredicate predicateWithFormat:@"name = %@", text];
    NSManagedObjectContext *context = [Utils getContext];
    NSError *error;
    NSArray *texts = [context executeFetchRequest:request error:&error];
    if(texts != nil && texts.count > 0) {
        NSString *text = ((Text *)[texts objectAtIndex:0]).value;
        if([control isKindOfClass:[UILabel class]]) {
            ((UILabel *)control).text = text;
        }
        else if([control isKindOfClass:[UITextField class]]) {
            ((UITextField *)control).placeholder = text;
        }
        else if([control isKindOfClass:[UIButton class]]) {
            [(UIButton *)control setTitle:text forState:UIControlStateNormal];
        }
    }
}

@end
