//
//  AboutViewController.m
//  MoveAdvisor
//
//  Created by Outsourcing IT Services Ltd. on 10/13/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "AboutViewController.h"
#import "Utils.h"
#import "User.h"
#import "URLRequests.h"

@interface AboutViewController ()

@property (weak, nonatomic) IBOutlet UIScrollView *scrollView;
@property (weak, nonatomic) IBOutlet UIView *viewContainer;
@property (weak, nonatomic) IBOutlet UIView *viewTable;

@property (weak, nonatomic) IBOutlet UILabel *labelTitle;
@property (weak, nonatomic) IBOutlet UIButton *buttonClose;

@property (weak, nonatomic) IBOutlet UITextView *labelText;

@property (weak, nonatomic) IBOutlet UIView *viewNameSeparator;
@property (weak, nonatomic) IBOutlet UILabel *labelNameHeader;
@property (weak, nonatomic) IBOutlet UILabel *labelNameError;
@property (weak, nonatomic) IBOutlet UITextField *textName;

@property (weak, nonatomic) IBOutlet UIView *viewEmailSeparator;
@property (weak, nonatomic) IBOutlet UILabel *labelEmailHeader;
@property (weak, nonatomic) IBOutlet UILabel *labelEmailError;
@property (weak, nonatomic) IBOutlet UITextField *textEmail;

@property (weak, nonatomic) IBOutlet UIView *viewMessageSeparator;
@property (weak, nonatomic) IBOutlet UILabel *labelMessageHeader;
@property (weak, nonatomic) IBOutlet UILabel *labelMessageError;
@property (weak, nonatomic) IBOutlet UITextField *textMessage;

@property (weak, nonatomic) IBOutlet UIButton *buttonSendMessage;

@property (assign, nonatomic) CGRect rect;

- (IBAction)onBack:(id)sender;
- (IBAction)onOK:(id)sender;
- (IBAction)onPrivacyPolicy:(id)sender;

@end

@implementation AboutViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.buttonSendMessage.layer.cornerRadius = 5;
    [self setSeparatorShadow:self.viewNameSeparator];
    [self setSeparatorShadow:self.viewEmailSeparator];
    [self setSeparatorShadow:self.viewMessageSeparator];
    User *user = [User getCurrent];
    self.textName.text = user.name;
    [self showLabel:self.labelNameHeader control:self.textName animated:NO];
    self.textEmail.text = user.email;
    [self showLabel:self.labelEmailHeader control:self.textEmail animated:NO];
    self.rect = CGRectZero;
    NSMutableAttributedString *text = [[NSMutableAttributedString alloc] init];
    [text appendAttributedString:[[NSAttributedString alloc] initWithString:@"Designed for anyone who’s planning to move house soon, MoveAdvisor is a powerful moving organization app that speeds up and facilitates the entire relocation process.\nMoving to "]];
    [text appendAttributedString:[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@, %@", [user.city objectForKey:@"name"], [user.state objectForKey:@"abr"]]]];
    [text appendAttributedString:[[NSAttributedString alloc] initWithString:@", can end up being a real challenge for you as a house move is often regarded as one of life’s most stressful events.\nLuckily, all of us from MoveAdvisor.com are dedicated to helping you have a smooth and problem-free moving experience from start to finish.\nIf you have any suggestions or ideas, or you just want to say hi, please use the form below to contact us.\nIn case you really like our app, please don’t forget to "]];
    NSMutableAttributedString *url = [[NSMutableAttributedString alloc] initWithString:@"rate"];
    [url addAttribute:NSLinkAttributeName value:@"https://itunes.apple.com/us/app/id1215040195" range:NSMakeRange(0, 4)];
    [url addAttribute:NSForegroundColorAttributeName value:[UIColor blueColor] range:NSMakeRange(0, 4)];
    [url addAttribute:NSUnderlineStyleAttributeName value:[NSNumber numberWithInt:NSUnderlineStyleSingle] range:NSMakeRange(0, 4)];
    [text appendAttributedString:url];
    [text appendAttributedString:[[NSAttributedString alloc] initWithString:@" it on the App Store so that other people can benefit from it as well.\nThank you for using MoveAdvisor!"]];
    [text addAttribute:NSFontAttributeName value:[UIFont systemFontOfSize:14] range:NSMakeRange(0, text.length)];
    self.labelText.attributedText = text;
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)setSeparatorShadow:(UIView *)viewSeparator {
    viewSeparator.layer.shadowOffset = CGSizeMake(0, 5);
    viewSeparator.layer.shadowColor = [Utils getColor:COLOR_LIGHT_GRAY].CGColor;
    viewSeparator.layer.shadowRadius = 5;
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(onShowKeyboard:) name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(onHideKeyboard:) name:UIKeyboardWillHideNotification object:nil];
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillHideNotification object:nil];
}

- (void)setConstraints:(int)height duration:(float)duration {
    NSLayoutConstraint *constraintBottom = nil;
    NSLayoutConstraint *constraintHeight = nil;
    for(NSLayoutConstraint *constraint in self.scrollView.constraints) {
        if(constraint.secondAttribute == NSLayoutAttributeBottom && constraint.secondItem == self.viewContainer) {
            constraintBottom = constraint;
        }
    }
    for(NSLayoutConstraint *constraint in self.viewContainer.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeHeight) {
            constraintHeight = constraint;
        }
    }
    CGFloat bottom;
    bottom = height;
    if(duration > 0) {
        [self.scrollView layoutIfNeeded];
        [UIView animateWithDuration:duration animations:^{
            constraintBottom.constant = height;
            [self.scrollView layoutIfNeeded];
        }];
    }
    else {
        constraintBottom.constant = height;
    }
}

- (void)viewDidLayoutSubviews {
    [super viewDidLayoutSubviews];
    if(!CGRectEqualToRect(self.rect, self.view.frame)) {
        [self setConstraints:0 duration:0];
        self.rect = self.view.frame;
    }
}

- (void)onShowKeyboard:(NSNotification *)notification {
    CGRect rect = ((NSValue *)[notification.userInfo objectForKey:UIKeyboardFrameEndUserInfoKey]).CGRectValue;
    double duration = ((NSNumber *)[notification.userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey]).doubleValue;
    [self setConstraints:rect.size.height duration:duration];
}

- (void)onHideKeyboard:(NSNotification *)notification {
    double duration = ((NSNumber *)[notification.userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey]).doubleValue;
    [self setConstraints:0 duration:duration];
}

/*
 #pragma mark - Navigation
 
 // In a storyboard-based application, you will often want to do a little preparation before navigation
 - (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
 // Get the new view controller using [segue destinationViewController].
 // Pass the selected object to the new view controller.
 }
 */

- (IBAction)onBack:(id)sender {
    //[self.view removeFromSuperview];
    //[self removeFromParentViewController];
    if([self.textName isFirstResponder]) {
        [self.textName resignFirstResponder];
    }
    if([self.textEmail isFirstResponder]) {
        [self.textEmail resignFirstResponder];
    }
    if([self.textMessage isFirstResponder]) {
        [self.textMessage resignFirstResponder];
    }
    [self dismissViewControllerAnimated:YES completion:^{
        
    }];
}

- (void)showLabel:(UILabel *)label control:(UIControl *)control animated:(BOOL)animated {
    if(animated) {
        CATransition *transition = [[CATransition alloc] init];
        transition.type = kCATransitionFade;
        transition.subtype = kCATransitionFromBottom;
        transition.duration = 0.25;
        [label.layer addAnimation:transition forKey:kCATransition];
    }
    label.hidden = NO;
    if([control isKindOfClass:[UIButton class]]) {
        [(UIButton *)control setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
    }
    for(NSLayoutConstraint *constraint in control.superview.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeTop && constraint.firstItem == control) {
            constraint.constant = 30;
        }
    }
}

- (void)hideLabel:(UILabel *)label control:(UIControl *)control animated:(BOOL)animated {
    if(animated) {
        CATransition *transition = [[CATransition alloc] init];
        transition.type = kCATransitionFade;
        transition.subtype = kCATransitionFromBottom;
        transition.duration = 0.25;
        [label.layer addAnimation:transition forKey:kCATransition];
    }
    label.hidden = YES;
    if([control isKindOfClass:[UIButton class]]) {
        [(UIButton *)control setTitleColor:[UIColor colorWithWhite:0.785f alpha:1] forState:UIControlStateNormal];
    }
    for(NSLayoutConstraint *constraint in control.superview.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeTop && constraint.firstItem == control) {
            constraint.constant = 5;
        }
    }
}

- (BOOL)check:(BOOL)valid label:(UILabel *)label {
    CATransition *transition = [[CATransition alloc] init];
    transition.type = kCATransitionFade;
    transition.subtype = kCATransitionFromBottom;
    transition.duration = 0.25;
    [label.layer addAnimation:transition forKey:kCATransition];
    if(valid) {
        label.hidden = YES;
        return YES;
    }
    else {
        label.hidden = NO;
        return NO;
    }
}

- (IBAction)onOK:(id)sender {
    BOOL checkName = [self check:[Utils validateName:self.textName.text] label:self.labelNameError];
    BOOL checkEmail = [self check:[Utils validateEmail:self.textEmail.text] label:self.labelEmailError];
    BOOL checkMessage = [self check:![self.textMessage.text isEqualToString:@""] label:self.labelMessageError];
    if(checkName && checkEmail && checkMessage) {
        User *user = [User getCurrent];
        user.name = self.textName.text;
        user.email = self.textEmail.text;
        [user save];
        if([self.textName isFirstResponder]) {
            [self.textName resignFirstResponder];
        }
        if([self.textEmail isFirstResponder]) {
            [self.textEmail resignFirstResponder];
        }
        if([self.textMessage isFirstResponder]) {
            [self.textMessage resignFirstResponder];
        }
        NSDictionary<NSString *, NSString *> *data = @{
                                                       @"name": self.textName.text,
                                                       @"email": self.textEmail.text,
                                                       @"message": self.textMessage.text
                                                       };
        UIViewController *controller = [[UIViewController alloc] init];
        controller.modalPresentationStyle = UIModalPresentationOverFullScreen;
        controller.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
        UIView *view = [[UIView alloc] init];
        view.backgroundColor = [Utils getColor:0x80000000];
        controller.view = view;
        UIActivityIndicatorView *progress = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhiteLarge];
        progress.color = [Utils getColor:COLOR_GREEN];
        progress.translatesAutoresizingMaskIntoConstraints = NO;
        [progress startAnimating];
        [view addSubview:progress];
        [view addConstraint:[NSLayoutConstraint constraintWithItem:progress attribute:NSLayoutAttributeCenterX relatedBy:NSLayoutRelationEqual toItem:view attribute:NSLayoutAttributeCenterX multiplier:1 constant:0]];
        [view addConstraint:[NSLayoutConstraint constraintWithItem:progress attribute:NSLayoutAttributeCenterY relatedBy:NSLayoutRelationEqual toItem:view attribute:NSLayoutAttributeCenterY multiplier:1 constant:0]];
        [self presentViewController:controller animated:YES completion:^{
            [URLRequests sendEmail:data to:@"about" receive:^(BOOL success, NSString *error){
                [self dismissViewControllerAnimated:YES completion:^{
                    if(success) {
                        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Message successfully sent!" message:@"Thank you! We appreciate your effort to provide us with feedback!" delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
                        [alert show];
                    }
                    else {
                        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Failed to send message!" message:error delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
                        [alert show];
                    }
                }];
            }];
        }];
    }
}

- (IBAction)onPrivacyPolicy:(id)sender {
    [[UIApplication sharedApplication] openURL:[NSURL URLWithString:@"https://www.moveadvisor.com/privacy-policy/"]];
}

#pragma mark - UIAlertViewDelegate

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
    [self.presentingViewController dismissViewControllerAnimated:YES completion:^{

    }];
}

#pragma mark - UITextFieldDelegate

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string {
    NSString *text = [textField.text stringByReplacingCharactersInRange:range withString:string];
    if(textField == self.textName) {
        if(![text isEqualToString:@""]) {
            [self showLabel:self.labelNameHeader control:self.textName animated:YES];
        }
        else {
            [self hideLabel:self.labelNameHeader control:self.textName animated:YES];
        }
        [self check:YES label:self.labelNameError];
    }
    else if(textField == self.textEmail) {
        if(![text isEqualToString:@""]) {
            [self showLabel:self.labelEmailHeader control:self.textEmail animated:YES];
        }
        else {
            [self hideLabel:self.labelEmailHeader control:self.textEmail animated:YES];
        }
        [self check:YES label:self.labelEmailError];
    }
    else if(textField == self.textMessage) {
        if(![text isEqualToString:@""]) {
            [self showLabel:self.labelMessageHeader control:self.textMessage animated:YES];
        }
        else {
            [self hideLabel:self.labelMessageHeader control:self.textMessage animated:YES];
        }
        [self check:YES label:self.labelMessageError];
    }
    return YES;
}

- (void)textFieldDidBeginEditing:(UITextField *)textField {
}

- (void)textFieldDidEndEditing:(UITextField *)textField {
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    [textField resignFirstResponder];
    return YES;
}

@end
