//
//  AdvertiseViewController.m
//  MoveAdvisor
//
//  Created by Outsourcing IT Services Ltd. on 10/13/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "AdvertiseViewController.h"
#import "Utils.h"
#import "User.h"
#import "URLRequests.h"

@interface AdvertiseViewController ()

@property (weak, nonatomic) IBOutlet UIScrollView *scrollView;
@property (weak, nonatomic) IBOutlet UIView *viewContainer;
@property (weak, nonatomic) IBOutlet UIView *viewTable;

@property (weak, nonatomic) IBOutlet UILabel *labelTitle;
@property (weak, nonatomic) IBOutlet UIButton *buttonClose;

@property (weak, nonatomic) IBOutlet UIView *viewCompanySeparator;
@property (weak, nonatomic) IBOutlet UILabel *labelCompanyHeader;
@property (weak, nonatomic) IBOutlet UILabel *labelCompanyError;
@property (weak, nonatomic) IBOutlet UITextField *textCompany;

@property (weak, nonatomic) IBOutlet UIView *viewNameSeparator;
@property (weak, nonatomic) IBOutlet UILabel *labelNameHeader;
@property (weak, nonatomic) IBOutlet UILabel *labelNameError;
@property (weak, nonatomic) IBOutlet UITextField *textName;

@property (weak, nonatomic) IBOutlet UIView *viewPhoneSeparator;
@property (weak, nonatomic) IBOutlet UILabel *labelPhoneHeader;
@property (weak, nonatomic) IBOutlet UILabel *labelPhoneError;
@property (weak, nonatomic) IBOutlet UITextField *textPhone;

@property (weak, nonatomic) IBOutlet UIView *viewEmailSeparator;
@property (weak, nonatomic) IBOutlet UILabel *labelEmailHeader;
@property (weak, nonatomic) IBOutlet UILabel *labelEmailError;
@property (weak, nonatomic) IBOutlet UITextField *textEmail;

@property (weak, nonatomic) IBOutlet UIView *viewWebsiteSeparator;
@property (weak, nonatomic) IBOutlet UILabel *labelWebsiteHeader;
@property (weak, nonatomic) IBOutlet UILabel *labelWebsiteError;
@property (weak, nonatomic) IBOutlet UITextField *textWebsite;

@property (weak, nonatomic) IBOutlet UIButton *buttonSendMessage;

@property (assign, nonatomic) CGRect rect;

- (IBAction)onBack:(id)sender;
- (IBAction)onOK:(id)sender;
- (IBAction)onPrivacyPolicy:(id)sender;

@end

@implementation AdvertiseViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.buttonSendMessage.layer.cornerRadius = 5;
    [self setSeparatorShadow:self.viewCompanySeparator];
    [self setSeparatorShadow:self.viewNameSeparator];
    [self setSeparatorShadow:self.viewPhoneSeparator];
    [self setSeparatorShadow:self.viewEmailSeparator];
    [self setSeparatorShadow:self.viewWebsiteSeparator];
    User *user = [User getCurrent];
    self.textCompany.text = user.company;
    [self showLabel:self.labelCompanyHeader control:self.textCompany animated:NO];
    self.textName.text = user.name;
    [self showLabel:self.labelNameHeader control:self.textName animated:NO];
    self.textPhone.text = user.phone;
    [self showLabel:self.labelPhoneHeader control:self.textPhone animated:NO];
    self.textEmail.text = user.email;
    [self showLabel:self.labelEmailHeader control:self.textEmail animated:NO];
    self.rect = CGRectZero;
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)setSeparatorShadow:(UIView *)viewSeparator {
    viewSeparator.layer.shadowOffset = CGSizeMake(0, 5);
    viewSeparator.layer.shadowColor = [Utils getColor:COLOR_LIGHT_GRAY].CGColor;
    viewSeparator.layer.shadowRadius = 5;
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(onShowKeyboard:) name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(onHideKeyboard:) name:UIKeyboardWillHideNotification object:nil];
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillHideNotification object:nil];
}

- (void)setConstraints:(int)height duration:(float)duration {
    NSLayoutConstraint *constraintBottom = nil;
    NSLayoutConstraint *constraintHeight = nil;
    for(NSLayoutConstraint *constraint in self.scrollView.constraints) {
        if(constraint.secondAttribute == NSLayoutAttributeBottom && constraint.secondItem == self.viewContainer) {
            constraintBottom = constraint;
        }
    }
    for(NSLayoutConstraint *constraint in self.viewContainer.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeHeight) {
            constraintHeight = constraint;
        }
    }
    CGFloat bottom;
    bottom = height;
    if(duration > 0) {
        [self.scrollView layoutIfNeeded];
        [UIView animateWithDuration:duration animations:^{
            constraintBottom.constant = height;
            [self.scrollView layoutIfNeeded];
        }];
    }
    else {
        constraintBottom.constant = height;
    }
}

- (void)viewDidLayoutSubviews {
    [super viewDidLayoutSubviews];
    if(!CGRectEqualToRect(self.rect, self.view.frame)) {
        [self setConstraints:0 duration:0];
        self.rect = self.view.frame;
    }
}

- (void)onShowKeyboard:(NSNotification *)notification {
    CGRect rect = ((NSValue *)[notification.userInfo objectForKey:UIKeyboardFrameEndUserInfoKey]).CGRectValue;
    double duration = ((NSNumber *)[notification.userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey]).doubleValue;
    [self setConstraints:rect.size.height duration:duration];
}

- (void)onHideKeyboard:(NSNotification *)notification {
    double duration = ((NSNumber *)[notification.userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey]).doubleValue;
    [self setConstraints:0 duration:duration];
}

/*
 #pragma mark - Navigation
 
 // In a storyboard-based application, you will often want to do a little preparation before navigation
 - (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
 // Get the new view controller using [segue destinationViewController].
 // Pass the selected object to the new view controller.
 }
 */

- (IBAction)onBack:(id)sender {
    if([self.textCompany isFirstResponder]) {
        [self.textCompany resignFirstResponder];
    }
    if([self.textName isFirstResponder]) {
        [self.textName resignFirstResponder];
    }
    if([self.textPhone isFirstResponder]) {
        [self.textPhone resignFirstResponder];
    }
    if([self.textEmail isFirstResponder]) {
        [self.textEmail resignFirstResponder];
    }
    if([self.textWebsite isFirstResponder]) {
        [self.textWebsite resignFirstResponder];
    }
    [self dismissViewControllerAnimated:YES completion:^{
        
    }];
}

- (void)showLabel:(UILabel *)label control:(UIControl *)control animated:(BOOL)animated {
    if(animated) {
        CATransition *transition = [[CATransition alloc] init];
        transition.type = kCATransitionFade;
        transition.subtype = kCATransitionFromBottom;
        transition.duration = 0.25;
        [label.layer addAnimation:transition forKey:kCATransition];
    }
    label.hidden = NO;
    if([control isKindOfClass:[UIButton class]]) {
        [(UIButton *)control setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
    }
    for(NSLayoutConstraint *constraint in control.superview.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeTop && constraint.firstItem == control) {
            constraint.constant = 30;
        }
    }
}

- (void)hideLabel:(UILabel *)label control:(UIControl *)control animated:(BOOL)animated {
    if(animated) {
        CATransition *transition = [[CATransition alloc] init];
        transition.type = kCATransitionFade;
        transition.subtype = kCATransitionFromBottom;
        transition.duration = 0.25;
        [label.layer addAnimation:transition forKey:kCATransition];
    }
    label.hidden = YES;
    if([control isKindOfClass:[UIButton class]]) {
        [(UIButton *)control setTitleColor:[UIColor colorWithWhite:0.785f alpha:1] forState:UIControlStateNormal];
    }
    for(NSLayoutConstraint *constraint in control.superview.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeTop && constraint.firstItem == control) {
            constraint.constant = 5;
        }
    }
}

- (BOOL)check:(BOOL)valid label:(UILabel *)label {
    CATransition *transition = [[CATransition alloc] init];
    transition.type = kCATransitionFade;
    transition.subtype = kCATransitionFromBottom;
    transition.duration = 0.25;
    [label.layer addAnimation:transition forKey:kCATransition];
    if(valid) {
        label.hidden = YES;
        return YES;
    }
    else {
        label.hidden = NO;
        return NO;
    }
}

- (IBAction)onOK:(id)sender {
    BOOL checkCompany = [self check:![self.textCompany.text isEqualToString:@""] label:self.labelCompanyError];
    BOOL checkName = [self check:[Utils validateName:self.textName.text] label:self.labelNameError];
    BOOL checkPhone = [self check:[Utils validatePhone:self.textPhone.text] label:self.labelPhoneError];
    BOOL checkEmail = [self check:[Utils validateEmail:self.textEmail.text] label:self.labelEmailError];
    BOOL checkWebsite = [self check:![self.textWebsite.text isEqualToString:@""] label:self.labelWebsiteError];
    if(checkCompany && checkName && checkPhone && checkEmail && checkWebsite) {
        User *user = [User getCurrent];
        user.company = self.textCompany.text;
        user.name = self.textName.text;
        user.email = self.textEmail.text;
        user.phone = self.textPhone.text;
        [user save];
        if([self.textCompany isFirstResponder]) {
            [self.textCompany resignFirstResponder];
        }
        if([self.textName isFirstResponder]) {
            [self.textName resignFirstResponder];
        }
        if([self.textPhone isFirstResponder]) {
            [self.textPhone resignFirstResponder];
        }
        if([self.textEmail isFirstResponder]) {
            [self.textEmail resignFirstResponder];
        }
        if([self.textWebsite isFirstResponder]) {
            [self.textWebsite resignFirstResponder];
        }
        NSDictionary<NSString *, NSString *> *data = @{
                                                       @"name": self.textName.text,
                                                       @"phone": self.textPhone.text,
                                                       @"email": self.textEmail.text,
                                                       @"company_name": self.textCompany.text,
                                                       @"company_website": self.textWebsite.text
                                                       };
        UIViewController *controller = [[UIViewController alloc] init];
        controller.modalPresentationStyle = UIModalPresentationOverFullScreen;
        controller.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
        UIView *view = [[UIView alloc] init];
        view.backgroundColor = [Utils getColor:0x80000000];
        controller.view = view;
        UIActivityIndicatorView *progress = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhiteLarge];
        progress.color = [Utils getColor:COLOR_GREEN];
        progress.translatesAutoresizingMaskIntoConstraints = NO;
        [progress startAnimating];
        [view addSubview:progress];
        [view addConstraint:[NSLayoutConstraint constraintWithItem:progress attribute:NSLayoutAttributeCenterX relatedBy:NSLayoutRelationEqual toItem:view attribute:NSLayoutAttributeCenterX multiplier:1 constant:0]];
        [view addConstraint:[NSLayoutConstraint constraintWithItem:progress attribute:NSLayoutAttributeCenterY relatedBy:NSLayoutRelationEqual toItem:view attribute:NSLayoutAttributeCenterY multiplier:1 constant:0]];
        [self presentViewController:controller animated:YES completion:^{
            [URLRequests sendEmail:data to:@"advertise" receive:^(BOOL success, NSString *error){
                [self dismissViewControllerAnimated:YES completion:^{
                    if(success) {
                        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Message successfully sent!" message:@"Thank you for contacting us. A representative will contact you soon. Thanks for reaching out." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
                        [alert show];
                    }
                    else {
                        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Failed to send message!" message:error delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
                        [alert show];
                    }
                }];
            }];
        }];
    }
}

- (IBAction)onPrivacyPolicy:(id)sender {
    [[UIApplication sharedApplication] openURL:[NSURL URLWithString:@"https://www.moveadvisor.com/privacy-policy/"]];
}

#pragma mark - UIAlertViewDelegate

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
    [self.presentingViewController dismissViewControllerAnimated:YES completion:^{

    }];
}

#pragma mark - UITextFieldDelegate

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string {
    NSString *text = [textField.text stringByReplacingCharactersInRange:range withString:string];
    if(textField == self.textCompany) {
        if(![text isEqualToString:@""]) {
            [self showLabel:self.labelCompanyHeader control:self.textCompany animated:YES];
        }
        else {
            [self hideLabel:self.labelCompanyHeader control:self.textCompany animated:YES];
        }
        [self check:YES label:self.labelCompanyError];
    }
    else if(textField == self.textName) {
        if(![text isEqualToString:@""]) {
            [self showLabel:self.labelNameHeader control:self.textName animated:YES];
        }
        else {
            [self hideLabel:self.labelNameHeader control:self.textName animated:YES];
        }
        [self check:YES label:self.labelNameError];
    }
    else if(textField == self.textPhone) {
        if(![text isEqualToString:@""]) {
            [self showLabel:self.labelPhoneHeader control:self.textPhone animated:YES];
        }
        else {
            [self hideLabel:self.labelPhoneHeader control:self.textPhone animated:YES];
        }
        [self check:YES label:self.labelPhoneError];
    }
    else if(textField == self.textEmail) {
        if(![text isEqualToString:@""]) {
            [self showLabel:self.labelEmailHeader control:self.textEmail animated:YES];
        }
        else {
            [self hideLabel:self.labelEmailHeader control:self.textEmail animated:YES];
        }
        [self check:YES label:self.labelEmailError];
    }
    else if(textField == self.textWebsite) {
        if(![text isEqualToString:@""]) {
            [self showLabel:self.labelWebsiteHeader control:self.textWebsite animated:YES];
        }
        else {
            [self hideLabel:self.labelWebsiteHeader control:self.textWebsite animated:YES];
        }
        [self check:YES label:self.labelWebsiteError];
    }
    return YES;
}

- (void)textFieldDidBeginEditing:(UITextField *)textField {
}

- (void)textFieldDidEndEditing:(UITextField *)textField {
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    [textField resignFirstResponder];
    return YES;
}

@end
