//
//  CalendarCollectionViewCell.m
//  MoveAdvisor
//
//  Created by Outsourcing IT Services Ltd. on 8/12/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "CalendarCollectionViewCell.h"
#import "Utils.h"

@interface CalendarCollectionViewCell ()

@property (strong, nonatomic) UILabel *labelText;

@property (assign, nonatomic) BOOL enabled;

@end

@implementation CalendarCollectionViewCell

- (instancetype)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        self.labelText = [[UILabel alloc] initWithFrame:frame];
        self.labelText.textAlignment = NSTextAlignmentCenter;
        self.labelText.font = [UIFont systemFontOfSize:20];
        self.labelText.clipsToBounds = YES;
        if(frame.size.width < frame.size.height) {
            self.labelText.layer.cornerRadius = frame.size.width / 2;
        }
        else {
            self.labelText.layer.cornerRadius = frame.size.height / 2;
        }
        [self.contentView addSubview:self.labelText];
    }
    return self;
}

- (void)layoutSubviews {
    [super layoutSubviews];
    CGRect frame = self.contentView.frame;
    self.labelText.frame = frame;
    if(frame.size.width < frame.size.height) {
        self.labelText.layer.cornerRadius = frame.size.width / 2;
    }
    else {
        self.labelText.layer.cornerRadius = frame.size.height / 2;
    }
}

- (void)setWeekDay:(NSString *)weekday {
    self.labelText.text = weekday;
    self.labelText.textColor = [UIColor whiteColor];
}

- (void)setDay:(long)day current:(BOOL)current selected:(BOOL)selected enabled:(BOOL)enabled {
    self.labelText.text = [NSString stringWithFormat:@"%ld", day];
    self.enabled = enabled;
    if(selected) {
        self.labelText.textColor = [UIColor whiteColor];
    }
    else if(enabled) {
        self.labelText.textColor = [UIColor darkGrayColor];
    }
    else {
        self.labelText.textColor = [UIColor lightGrayColor];
    }
    if(selected) {
        self.labelText.backgroundColor = [Utils getColor:COLOR_GREEN];
    }
    else if(current) {
        self.labelText.backgroundColor = [Utils getColor:COLOR_LIGHT_GRAY];
    }
    else {
        self.labelText.backgroundColor = [UIColor clearColor];
    }
}

- (void)setHighlighted:(BOOL)highlighted {
    [super setHighlighted:highlighted];
    if(highlighted) {
        self.labelText.textColor = [UIColor whiteColor];
        self.labelText.backgroundColor = [Utils getColor:COLOR_GREEN];
    }
    else {
        if(self.selected) {
            self.labelText.textColor = [UIColor whiteColor];
            self.labelText.backgroundColor = [Utils getColor:COLOR_GREEN];
        }
        else {
            self.labelText.textColor = [UIColor darkGrayColor];
            self.labelText.backgroundColor = [UIColor clearColor];
        }
    }
}

- (void)setSelected:(BOOL)selected {
    [super setSelected:selected];
    if(self.selected) {
        self.labelText.textColor = [UIColor whiteColor];
        self.labelText.backgroundColor = [Utils getColor:COLOR_GREEN];
    }
    else {
        if(self.enabled) {
            self.labelText.textColor = [UIColor darkGrayColor];
        }
        else {
            self.labelText.textColor = [UIColor grayColor];
        }
        self.labelText.backgroundColor = [UIColor clearColor];
    }
}

@end
