//
//  CalendarCollectionViewController.m
//  MoveAdvisor
//
//  Created by Outsourcing IT Services Ltd. on 8/12/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "CalendarCollectionViewController.h"
#import "CalendarCollectionViewCell.h"
#import "CalendarViewController.h"

@interface CalendarCollectionViewController ()

@property (assign, nonatomic) NSInteger first;
@property (assign, nonatomic) NSInteger count;
@property (assign, nonatomic) NSInteger prevcount;

@property (strong, nonatomic) UICollectionView *collection;

@property (strong, nonatomic) NSCalendar *calendar;

@property (strong, nonatomic) NSDate *dateMin;
@property (strong, nonatomic) NSDate *dateMax;
@property (strong, nonatomic) NSDate *dateCurrent;
@property (strong, nonatomic) NSDate *dateSelected;

@end

@implementation CalendarCollectionViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc] init];
    int size = (self.view.frame.size.width - 20 + 10) / 7 - 10;
    layout.itemSize = CGSizeMake(size, size);
    layout.minimumInteritemSpacing = 10;
    layout.minimumLineSpacing = 10;
    int insetLeft = (self.view.frame.size.width - (size + 10) * 7 + 10) / 2;
    int insetRight = self.view.frame.size.width - (size + 10) * 7 + 10 - insetLeft;
    layout.sectionInset = UIEdgeInsetsMake(insetLeft / 2, insetLeft, insetRight / 2, insetRight);
    self.collection = [[UICollectionView alloc] initWithFrame:self.view.bounds collectionViewLayout:layout];
    [self.collection registerClass:[CalendarCollectionViewCell class] forCellWithReuseIdentifier:@"cell"];
    [self.collection registerClass:[CalendarCollectionViewCell class] forCellWithReuseIdentifier:@"title"];
    self.collection.dataSource = self;
    self.collection.delegate = self;
    self.collection.backgroundColor = [UIColor clearColor];
    [self.view addSubview:self.collection];
    self.view.backgroundColor = [UIColor clearColor];
    self.calendar = [NSCalendar calendarWithIdentifier:NSCalendarIdentifierGregorian];
    self.calendar.locale = [NSLocale localeWithLocaleIdentifier:@"en_us"];
    NSDateComponents *components = [[NSDateComponents alloc] init];
    components.year = self.year;
    components.month = self.month;
    components.day = 1;
    NSDate *date = [self.calendar dateFromComponents:components];
    self.first = [self.calendar component:NSCalendarUnitWeekday fromDate:date];
    NSRange range = [self.calendar rangeOfUnit:NSCalendarUnitDay inUnit:NSCalendarUnitMonth forDate:date];
    self.count = range.length;
    date = [self.calendar dateByAddingUnit:NSCalendarUnitDay value:-1 toDate:date options:0];
    self.prevcount = [self.calendar component:NSCalendarUnitDay fromDate:date];
    date = [NSDate date];
    components = [self.calendar components:NSCalendarUnitYear | NSCalendarUnitMonth | NSCalendarUnitDay fromDate:date];
    date = [self.calendar dateFromComponents:components];
    self.dateMin = [self.calendar dateByAddingUnit:NSCalendarUnitDay value:1 toDate:date options:0];
    self.dateMax = [self.calendar dateByAddingUnit:NSCalendarUnitMonth value:6 toDate:date options:0];
    self.dateCurrent = date;
    components.year = [self.date objectForKey:@"year"].intValue;
    components.month = [self.date objectForKey:@"month"].intValue;
    components.day = [self.date objectForKey:@"day"].intValue;
    self.dateSelected = [self.calendar dateFromComponents:components];
}

- (void)viewWillLayoutSubviews {
    [super viewWillLayoutSubviews];
    self.collection.frame = self.view.frame;
    UICollectionViewFlowLayout *layout = (UICollectionViewFlowLayout *)self.collection.collectionViewLayout;
    int size = (self.view.frame.size.width - 20 + 10) / 7 - 10;
    layout.itemSize = CGSizeMake(size, size);
    int insetLeft = (self.view.frame.size.width - (size + 10) * 7 + 10) / 2;
    int insetRight = self.view.frame.size.width - (size + 10) * 7 + 10 - insetLeft;
    layout.sectionInset = UIEdgeInsetsMake(insetLeft / 2, insetLeft, insetRight / 2, insetRight);
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

#pragma mark - UICollectionViewDataSource

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView {
    return 2;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    if(section == 0) {
        return 7;
    }
    else {
        return (self.first - 1 + self.count + 6) / 7 * 7;
    }
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    if(indexPath.section == 0) {
        CalendarCollectionViewCell *cell = (CalendarCollectionViewCell *)[collectionView dequeueReusableCellWithReuseIdentifier:@"title" forIndexPath:indexPath];
        [cell setWeekDay:[self.calendar.veryShortWeekdaySymbols objectAtIndex:indexPath.item]];
        return cell;
    }
    else {
        CalendarCollectionViewCell *cell = (CalendarCollectionViewCell *)[collectionView dequeueReusableCellWithReuseIdentifier:@"cell" forIndexPath:indexPath];
        NSDateComponents *components = [[NSDateComponents alloc] init];
        components.year = self.year;
        components.month = self.month;
        components.day = indexPath.item - self.first + 1 + 1;
        NSDate *date = [self.calendar dateFromComponents:components];
        BOOL current = [date compare:self.dateCurrent] == NSOrderedSame;
        BOOL selected = [date compare:self.dateSelected] == NSOrderedSame;
        BOOL enabled = [date compare:self.dateMin] != NSOrderedAscending && [date compare:self.dateMax] != NSOrderedDescending;
        if(indexPath.item < self.first - 1) {
            [cell setDay:self.prevcount - self.first + 1 + indexPath.item + 1 current:current selected:selected enabled:NO];
        }
        else if(indexPath.item < self.first + self.count - 1) {
            [cell setDay:indexPath.item - self.first + 1 + 1 current:current selected:selected enabled:enabled];
        }
        else {
            [cell setDay:indexPath.item - self.first - self.count + 1 + 1 current:current selected:selected enabled:NO];
        }
        return cell;
    }
}

#pragma mark - UICollectionViewDelegate

- (BOOL)collectionView:(UICollectionView *)collectionView shouldHighlightItemAtIndexPath:(NSIndexPath *)indexPath {
    if(indexPath.section == 0) {
        return NO;
    }
    else {
        if(indexPath.item < self.first - 1) {
            return NO;
        }
        else if(indexPath.item < self.first + self.count - 1) {
            NSDateComponents *components = [[NSDateComponents alloc] init];
            components.year = self.year;
            components.month = self.month;
            components.day = indexPath.item - self.first + 1 + 1;
            NSDate *date = [self.calendar dateFromComponents:components];
            BOOL enabled = [date compare:self.dateMin] != NSOrderedAscending && [date compare:self.dateMax] != NSOrderedDescending;
            return enabled;
        }
        else {
            return NO;
        }
    }
}

- (BOOL)collectionView:(UICollectionView *)collectionView shouldSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    if(indexPath.section == 0) {
        return NO;
    }
    else {
        if(indexPath.item < self.first - 1) {
            return NO;
        }
        else if(indexPath.item < self.first + self.count - 1) {
            NSDateComponents *components = [[NSDateComponents alloc] init];
            components.year = self.year;
            components.month = self.month;
            components.day = indexPath.item - self.first + 1 + 1;
            NSDate *date = [self.calendar dateFromComponents:components];
            BOOL enabled = [date compare:self.dateMin] != NSOrderedAscending && [date compare:self.dateMax] != NSOrderedDescending;
            return enabled;
        }
        else {
            return NO;
        }
    }
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    if(indexPath.section == 0) {
    }
    else {
        if(indexPath.item < self.first - 1) {
        }
        else if(indexPath.item < self.first + self.count - 1) {
            self.day = indexPath.item - self.first + 1 + 1;
            if(self.owner != nil) {
                [self.owner onChange:self];
            }
        }
        else {
        }
    }
}

@end
