//
//  CalendarViewController.m
//  MoveAdvisor
//
//  Created by Outsourcing IT Services Ltd. on 8/13/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "CalendarViewController.h"
#import "CalendarCollectionViewController.h"

@interface CalendarViewController ()

@property (weak, nonatomic) IBOutlet UIView *viewContainer;
@property (weak, nonatomic) IBOutlet UILabel *labelTitle;
@property (weak, nonatomic) IBOutlet UIView *viewTitle;

@property (weak, nonatomic) UIPageViewController *pager;

@property (strong, nonatomic) NSCalendar *calendar;
@property (strong, nonatomic) NSDateComponents *components;

- (IBAction)onBack:(id)sender;
- (IBAction)onPrev:(id)sender;
- (IBAction)onNext:(id)sender;

@end

@implementation CalendarViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.calendar = [NSCalendar calendarWithIdentifier:NSCalendarIdentifierGregorian];
    self.calendar.locale = [NSLocale localeWithLocaleIdentifier:@"en_us"];
    self.viewContainer.layer.cornerRadius = 5;
    NSCalendar *current = [NSCalendar currentCalendar];
    self.components = [[NSDateComponents alloc] init];
    if(self.date != nil) {
        self.components.year = [self.date objectForKey:@"year"].intValue;
        self.components.month = [self.date objectForKey:@"month"].intValue;
        self.components.day = [self.date objectForKey:@"day"].intValue;
    }
    else {
        NSDate *date = [NSDate date];
        NSDateComponents *components = [current components:NSCalendarUnitYear | NSCalendarUnitMonth | NSCalendarUnitDay fromDate:date];
        self.components.year = components.year;
        self.components.month = components.month;
        self.components.day = components.day;
    }
    self.labelTitle.text = [self.calendar.monthSymbols objectAtIndex:self.components.month - 1];
    CalendarCollectionViewController *calendar = [[CalendarCollectionViewController alloc] init];
    calendar.owner = self;
    calendar.year = self.components.year;
    calendar.month = self.components.month;
    calendar.day = self.components.day;
    calendar.date = self.date;
    [self.pager setViewControllers:@[calendar] direction:UIPageViewControllerNavigationDirectionForward animated:NO completion:^(BOOL finished) {
        
    }];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
    if([segue.destinationViewController isKindOfClass:[UIPageViewController class]]) {
        self.pager = (UIPageViewController *)segue.destinationViewController;
        self.pager.dataSource = self;
        self.pager.delegate = self;
        
    }
}

- (void)viewWillLayoutSubviews {
    [super viewWillLayoutSubviews];
    for(NSLayoutConstraint *constraint in self.viewContainer.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeHeight) {
            constraint.constant = self.view.frame.size.width - 20 + 70;
        }
    }
    for(NSLayoutConstraint *constraint in self.viewTitle.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeHeight) {
            int size = (self.viewContainer.frame.size.width - 20 + 10) / 7 - 10;
            int inset = (self.viewContainer.frame.size.width - (size + 10) * 7 + 10) / 2;
            constraint.constant = size + inset;
        }
    }
}


- (IBAction)onBack:(id)sender {
    [self dismissViewControllerAnimated:YES completion:^{
        
    }];
}

- (IBAction)onPrev:(id)sender {
    if(self.pager.viewControllers.count == 1) {
        CalendarCollectionViewController *current = (CalendarCollectionViewController *)[self.pager.viewControllers objectAtIndex:0];
        CalendarCollectionViewController *controller = (CalendarCollectionViewController *)[self pageViewController:self.pager viewControllerBeforeViewController:current];
        [self.pager setViewControllers:@[controller] direction:UIPageViewControllerNavigationDirectionReverse animated:YES completion:^(BOOL finished) {
            if(self.pager.viewControllers.count == 1) {
                CalendarCollectionViewController *calendar = (CalendarCollectionViewController *)[self.pager.viewControllers objectAtIndex:0];
                self.labelTitle.text = [self.calendar.monthSymbols objectAtIndex:calendar.month - 1];
            }
        }];
    }
}

- (IBAction)onNext:(id)sender {
    if(self.pager.viewControllers.count == 1) {
        CalendarCollectionViewController *current = (CalendarCollectionViewController *)[self.pager.viewControllers objectAtIndex:0];
        CalendarCollectionViewController *controller = (CalendarCollectionViewController *)[self pageViewController:self.pager viewControllerAfterViewController:current];
        [self.pager setViewControllers:@[controller] direction:UIPageViewControllerNavigationDirectionForward animated:YES completion:^(BOOL finished) {
            if(self.pager.viewControllers.count == 1) {
                CalendarCollectionViewController *calendar = (CalendarCollectionViewController *)[self.pager.viewControllers objectAtIndex:0];
                self.labelTitle.text = [self.calendar.monthSymbols objectAtIndex:calendar.month - 1];
            }
        }];
    }
}

- (void)onChange:(CalendarCollectionViewController *)controller {
    self.components.year = controller.year;
    self.components.month = controller.month;
    self.components.day = controller.day;
    if(self.close != nil) {
        NSDictionary *date = @{@"year": [NSNumber numberWithLong:controller.year], @"month": [NSNumber numberWithLong:controller.month], @"day": [NSNumber numberWithLong:controller.day]};
        self.close(date);
    }
    [self dismissViewControllerAnimated:YES completion:^{
        
    }];
}

#pragma mark - UIPageViewControllerDataSource

- (UIViewController *)pageViewController:(UIPageViewController *)pageViewController viewControllerBeforeViewController:(UIViewController *)viewController {
    CalendarCollectionViewController *current = (CalendarCollectionViewController *)viewController;
    NSDateComponents *components = [[NSDateComponents alloc] init];
    components.year = current.year;
    components.month = current.month;
    components.day = 1;
    NSDate *date = [self.calendar dateFromComponents:components];
    date = [self.calendar dateByAddingUnit:NSCalendarUnitMonth value:-1 toDate:date options:0];
    components = [self.calendar components:NSCalendarUnitYear | NSCalendarUnitMonth | NSCalendarUnitDay fromDate:date];
    CalendarCollectionViewController *calendar = [[CalendarCollectionViewController alloc] init];
    calendar.owner = self;
    calendar.year = components.year;
    calendar.month = components.month;
    calendar.day = 0;
    calendar.date = self.date;
    return calendar;
}

- (UIViewController *)pageViewController:(UIPageViewController *)pageViewController viewControllerAfterViewController:(UIViewController *)viewController {
    CalendarCollectionViewController *current = (CalendarCollectionViewController *)viewController;
    NSDateComponents *components = [[NSDateComponents alloc] init];
    components.year = current.year;
    components.month = current.month;
    components.day = 1;
    NSDate *date = [self.calendar dateFromComponents:components];
    date = [self.calendar dateByAddingUnit:NSCalendarUnitMonth value:1 toDate:date options:0];
    components = [self.calendar components:NSCalendarUnitYear | NSCalendarUnitMonth | NSCalendarUnitDay fromDate:date];
    CalendarCollectionViewController *calendar = [[CalendarCollectionViewController alloc] init];
    calendar.owner = self;
    calendar.year = components.year;
    calendar.month = components.month;
    calendar.day = 0;
    calendar.date = self.date;
    return calendar;
}

#pragma mark - UIPageViewControllerDelegate

- (void)pageViewController:(UIPageViewController *)pageViewController didFinishAnimating:(BOOL)finished previousViewControllers:(NSArray<UIViewController *> *)previousViewControllers transitionCompleted:(BOOL)completed {
    if(self.pager.viewControllers.count == 1) {
        CalendarCollectionViewController *calendar = (CalendarCollectionViewController *)[self.pager.viewControllers objectAtIndex:0];
        self.labelTitle.text = [self.calendar.monthSymbols objectAtIndex:calendar.month - 1];
    }
}

@end
