//
//  MoveRideViewController.m
//  RideShipper
//
//  Created by Outsourcing It Services Ltd on 2/14/17.
//  Copyright © 2017 Outsourcing IT Services Ltd. All rights reserved.
//

#import "MoveRideViewController.h"
#import "MoveUserViewController.h"
#import "RideListViewController.h"
#import "RideImageListViewController.h"
#import "User.h"
#import "Utils.h"

@interface MoveRideViewController ()

@property (weak, nonatomic) IBOutlet UIImageView *imageBackground;

@property (weak, nonatomic) IBOutlet UIScrollView *scrollView;
@property (weak, nonatomic) IBOutlet UIView *viewContainer;
@property (weak, nonatomic) IBOutlet UIView *viewTable;

@property (weak, nonatomic) IBOutlet UILabel *labelTitle;
@property (weak, nonatomic) IBOutlet UILabel *labelSubTitle;
@property (weak, nonatomic) IBOutlet UIButton *buttonClose;

@property (weak, nonatomic) IBOutlet UILabel *labelRideMakeHeader;
@property (weak, nonatomic) IBOutlet UILabel *labelRideMakeError;
@property (weak, nonatomic) IBOutlet UIButton *buttonRideMake;

@property (weak, nonatomic) IBOutlet UILabel *labelRideModelHeader;
@property (weak, nonatomic) IBOutlet UILabel *labelRideModelError;
@property (weak, nonatomic) IBOutlet UIButton *buttonRideModel;

@property (weak, nonatomic) IBOutlet UILabel *labelRideConditionHeader;
@property (weak, nonatomic) IBOutlet UILabel *labelRideConditionError;
@property (weak, nonatomic) IBOutlet UIButton *buttonRideCondition;

@property (weak, nonatomic) IBOutlet UILabel *labelShippingMethodHeader;
@property (weak, nonatomic) IBOutlet UILabel *labelShippingMethodError;
@property (weak, nonatomic) IBOutlet UIButton *buttonShippingMethod;

@property (weak, nonatomic) IBOutlet UIButton *buttonOK;

@property (weak, nonatomic) IBOutlet UIView *viewPager;

@property (strong, nonatomic) NSDictionary<NSString *, NSString *> *make;
@property (strong, nonatomic) NSDictionary<NSString *, NSString *> *model;
@property (strong, nonatomic) NSDictionary<NSString *, NSString *> *condition;
@property (strong, nonatomic) NSDictionary<NSString *, NSString *> *method;

@property (strong, nonatomic) User *user;

@property (assign, nonatomic) CGRect rect;

- (IBAction)onBack:(id)sender;

@end

@implementation MoveRideViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    if(self.exit) {
        self.imageBackground.hidden = YES;
    }
    self.viewTable.layer.cornerRadius = 5;
    [Utils setText:self.labelTitle text:@"rideform-heading"];
    [Utils setText:self.labelSubTitle text:@"rideform-description_text"];
    [Utils setText:self.labelRideMakeHeader text:@"rideform-make"];
    [Utils setText:self.buttonRideMake text:@"rideform-make"];
    [Utils setText:self.labelRideModelHeader text:@"rideform-model"];
    [Utils setText:self.buttonRideModel text:@"rideform-model"];
    [Utils setText:self.labelRideConditionHeader text:@"rideform-condition"];
    [Utils setText:self.buttonRideCondition text:@"rideform-condition"];
    [Utils setText:self.labelShippingMethodHeader text:@"rideform-method"];
    [Utils setText:self.buttonShippingMethod text:@"rideform-method"];
    if(!self.exit) {
        [Utils setText:self.buttonOK text:@"rideform-button_submit"];
    }
    else {
        [self.buttonOK setTitle:@"Save changes" forState:UIControlStateNormal];
        self.viewPager.hidden = YES;
        NSLayoutConstraint *constraintHeight = nil;
        NSLayoutConstraint *constraintBottom = nil;
        for(NSLayoutConstraint *constraint in self.viewContainer.constraints) {
            if(constraint.firstAttribute == NSLayoutAttributeHeight) {
                constraintHeight = constraint;
            }
            else if(constraint.secondAttribute == NSLayoutAttributeBottom && constraint.secondItem == self.viewTable) {
                constraintBottom = constraint;
            }
        }
        constraintHeight.constant -= constraintBottom.constant - 10;
        constraintBottom.constant = 10;
    }
    self.user = [User getCurrent];
    if(self.user != nil) {
        if(self.user.move) {
            self.make = self.user.make;
            [self.buttonRideMake setTitle:(NSString *)[self.make objectForKey:@"make_name"] forState:UIControlStateNormal];
            [self showLabel:self.labelRideMakeHeader control:self.buttonRideMake animated:NO];
            self.model = self.user.model;
            [self.buttonRideModel setTitle:(NSString *)[self.model objectForKey:@"model_name"] forState:UIControlStateNormal];
            [self showLabel:self.labelRideModelHeader control:self.buttonRideModel animated:NO];
            self.condition = self.user.condition;
            [self.buttonRideCondition setTitle:(NSString *)[self.condition objectForKey:@"name"] forState:UIControlStateNormal];
            [self showLabel:self.labelRideConditionHeader control:self.buttonRideCondition animated:NO];
            self.method = self.user.method;
            [self.buttonShippingMethod setTitle:(NSString *)[self.method objectForKey:@"name"] forState:UIControlStateNormal];
            [self showLabel:self.labelShippingMethodHeader control:self.buttonShippingMethod animated:NO];
        }
    }
    else {
        self.user = [[User alloc] init];
    }
    self.buttonClose.hidden = !self.user.move;
    self.rect = CGRectZero;
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(onShowKeyboard:) name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(onHideKeyboard:) name:UIKeyboardWillHideNotification object:nil];
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillHideNotification object:nil];
}

- (void)setConstraints:(int)height duration:(float)duration {
    NSLayoutConstraint *constraintTop = nil;
    NSLayoutConstraint *constraintBottom = nil;
    NSLayoutConstraint *constraintHeight = nil;
    for(NSLayoutConstraint *constraint in self.scrollView.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeTop && constraint.firstItem == self.viewContainer) {
            constraintTop = constraint;
        }
        else if(constraint.secondAttribute == NSLayoutAttributeBottom && constraint.secondItem == self.viewContainer) {
            constraintBottom = constraint;
        }
    }
    for(NSLayoutConstraint *constraint in self.viewContainer.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeHeight) {
            constraintHeight = constraint;
        }
    }
    CGFloat top;
    CGFloat bottom;
    if(self.scrollView.frame.size.height - height - constraintHeight.constant - [self.topLayoutGuide length] > 0) {
        top = (self.scrollView.frame.size.height - height - constraintHeight.constant - [self.topLayoutGuide length]) / 2 + [self.topLayoutGuide length];
        bottom = self.scrollView.frame.size.height - constraintHeight.constant - top;
    }
    else {
        top = [self.topLayoutGuide length];
        bottom = height;
    }
    if(duration > 0) {
        [self.scrollView layoutIfNeeded];
        [UIView animateWithDuration:duration animations:^{
            constraintTop.constant = top;
            constraintBottom.constant = bottom;
            [self.scrollView layoutIfNeeded];
        }];
    }
    else {
        constraintTop.constant = top;
        constraintBottom.constant = bottom;
    }
}

- (void)viewDidLayoutSubviews {
    [super viewDidLayoutSubviews];
    if(!CGRectEqualToRect(self.rect, self.view.frame)) {
        [self setConstraints:0 duration:0];
        self.rect = self.view.frame;
    }
}

- (void)onShowKeyboard:(NSNotification *)notification {
    CGRect rect = ((NSValue *)[notification.userInfo objectForKey:UIKeyboardFrameEndUserInfoKey]).CGRectValue;
    double duration = ((NSNumber *)[notification.userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey]).doubleValue;
    [self setConstraints:rect.size.height duration:duration];
}

- (void)onHideKeyboard:(NSNotification *)notification {
    double duration = ((NSNumber *)[notification.userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey]).doubleValue;
    [self setConstraints:0 duration:duration];
}

#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
    if(sender == self.buttonRideMake) {
        ((RideListViewController *)segue.destinationViewController).type = @"ride make";
        ((RideListViewController *)segue.destinationViewController).list = @"makes";
        ((RideListViewController *)segue.destinationViewController).value = self.make;
        ((RideListViewController *)segue.destinationViewController).values = self.makes;
        ((RideListViewController *)segue.destinationViewController).close = ^(NSDictionary *make) {
            if(self.make == nil || [(NSNumber *)[self.make objectForKey:@"make_id"] compare:(NSNumber *)[make objectForKey:@"make_id"]] != NSOrderedSame) {
                self.model = nil;
                [self.buttonRideModel setTitle:@"Ride model" forState:UIControlStateNormal];
                [self hideLabel:self.labelRideModelHeader control:self.buttonRideModel animated:YES];
            }
            self.make = make;
            [self.buttonRideMake setTitle:[self.make objectForKey:@"make_name"] forState:UIControlStateNormal];
            [self showLabel:self.labelRideMakeHeader control:self.buttonRideMake animated:NO];
            [self check:self.make label:self.labelRideMakeError];
        };
    }
    else if(sender == self.buttonRideModel) {
        ((RideListViewController *)segue.destinationViewController).type = @"ride model";
        ((RideListViewController *)segue.destinationViewController).list = @"models";
        ((RideListViewController *)segue.destinationViewController).value = self.model;
        ((RideListViewController *)segue.destinationViewController).filter = self.make;
        ((RideListViewController *)segue.destinationViewController).close = ^(NSDictionary *model) {
            self.model = model;
            [self.buttonRideModel setTitle:[self.model objectForKey:@"model_name"] forState:UIControlStateNormal];
            [self showLabel:self.labelRideModelHeader control:self.buttonRideModel animated:NO];
            [self check:self.model label:self.labelRideModelError];
        };
    }
    else if(sender == self.buttonRideCondition) {
        ((RideImageListViewController *)segue.destinationViewController).image = @"ride-condition";
        ((RideImageListViewController *)segue.destinationViewController).type = @"ride condition";
        ((RideImageListViewController *)segue.destinationViewController).list = @[@{@"name": @"Running", @"image": @"running"}, @{@"name": @"Not running", @"image": @"not-running"}];
        ((RideImageListViewController *)segue.destinationViewController).value = self.condition;
        ((RideImageListViewController *)segue.destinationViewController).close = ^(NSDictionary *condition) {
            self.condition = condition;
            [self.buttonRideCondition setTitle:[self.condition objectForKey:@"name"] forState:UIControlStateNormal];
            [self showLabel:self.labelRideConditionHeader control:self.buttonRideCondition animated:NO];
            [self check:self.condition label:self.labelRideConditionError];
        };
    }
    else if(sender == self.buttonShippingMethod) {
        ((RideImageListViewController *)segue.destinationViewController).image = @"shipping-method";
        ((RideImageListViewController *)segue.destinationViewController).type = @"shipping method";
        ((RideImageListViewController *)segue.destinationViewController).list = @[@{@"name": @"Open", @"image": @"open-shipping"}, @{@"name": @"Enclosed", @"image": @"enclosed-shipping"}];
        ((RideImageListViewController *)segue.destinationViewController).value = self.method;
        ((RideImageListViewController *)segue.destinationViewController).close = ^(NSDictionary *method) {
            self.method = method;
            [self.buttonShippingMethod setTitle:[self.method objectForKey:@"name"] forState:UIControlStateNormal];
            [self showLabel:self.labelShippingMethodHeader control:self.buttonShippingMethod animated:NO];
            [self check:self.method label:self.labelShippingMethodError];
        };
    }
}


- (IBAction)onBack:(id)sender {
    [self dismissViewControllerAnimated:YES completion:^{
        
    }];
}

- (void)showLabel:(UILabel *)label control:(UIControl *)control animated:(BOOL)animated {
    if(animated) {
        CATransition *transition = [[CATransition alloc] init];
        transition.type = kCATransitionFade;
        transition.subtype = kCATransitionFromBottom;
        transition.duration = 0.25;
        [label.layer addAnimation:transition forKey:kCATransition];
    }
    label.hidden = NO;
    if([control isKindOfClass:[UIButton class]]) {
        [(UIButton *)control setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
    }
    for(NSLayoutConstraint *constraint in control.superview.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeTop && constraint.firstItem == control) {
            constraint.constant = 30;
        }
    }
}

- (void)hideLabel:(UILabel *)label control:(UIControl *)control animated:(BOOL)animated {
    if(animated) {
        CATransition *transition = [[CATransition alloc] init];
        transition.type = kCATransitionFade;
        transition.subtype = kCATransitionFromBottom;
        transition.duration = 0.25;
        [label.layer addAnimation:transition forKey:kCATransition];
    }
    label.hidden = YES;
    if([control isKindOfClass:[UIButton class]]) {
        [(UIButton *)control setTitleColor:[UIColor colorWithWhite:0.785f alpha:1] forState:UIControlStateNormal];
    }
    for(NSLayoutConstraint *constraint in control.superview.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeTop && constraint.firstItem == control) {
            constraint.constant = 5;
        }
    }
}

- (BOOL)check:(NSDictionary *)obj label:(UILabel *)label {
    CATransition *transition = [[CATransition alloc] init];
    transition.type = kCATransitionFade;
    transition.subtype = kCATransitionFromBottom;
    transition.duration = 0.25;
    [label.layer addAnimation:transition forKey:kCATransition];
    if(obj != nil) {
        label.hidden = YES;
        return YES;
    }
    else {
        label.hidden = NO;
        return NO;
    }
}

- (IBAction)onOK:(id)sender {
    BOOL checkMake = [self check:self.make label:self.labelRideMakeError];
    BOOL checkModel = [self check:self.model label:self.labelRideModelError];
    BOOL checkCondition= [self check:self.condition label:self.labelRideConditionError];
    BOOL checkMethod = [self check:self.method label:self.labelShippingMethodError];
    if(checkMake && checkModel && checkCondition && checkMethod) {
        self.user.make = self.make;
        self.user.model = self.model;
        self.user.condition = self.condition;
        self.user.method = self.method;
        self.user.move = YES;
        [self.user save];
        if(self.exit) {
            [self dismissViewControllerAnimated:YES completion:^{
                if(self.close != nil) {
                    self.close();
                }
            }];
        }
        else {
            [self performSegueWithIdentifier:@"user" sender:self];
        }
    }
}

@end
