//
//  MoveUserViewController.m
//  MoveAdvisor
//
//  Created by Outsourcing IT Services Ltd. on 8/30/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "MoveUserViewController.h"
#import "User.h"
#import "Utils.h"
#import "URLRequests.h"

@interface MoveUserViewController ()

@property (weak, nonatomic) IBOutlet UIImageView *imageBackground;

@property (weak, nonatomic) IBOutlet UIScrollView *scrollView;
@property (weak, nonatomic) IBOutlet UIView *viewContainer;
@property (weak, nonatomic) IBOutlet UIView *viewTable;

@property (weak, nonatomic) IBOutlet UILabel *labelTitle;
@property (weak, nonatomic) IBOutlet UILabel *labelSubTitle;
@property (weak, nonatomic) IBOutlet UIButton *buttonClose;

@property (weak, nonatomic) IBOutlet UILabel *labelNameHeader;
@property (weak, nonatomic) IBOutlet UILabel *labelNameError;
@property (weak, nonatomic) IBOutlet UITextField *textName;

@property (weak, nonatomic) IBOutlet UILabel *labelPhoneHeader;
@property (weak, nonatomic) IBOutlet UILabel *labelPhoneError;
@property (weak, nonatomic) IBOutlet UITextField *textPhone;

@property (weak, nonatomic) IBOutlet UILabel *labelEmailHeader;
@property (weak, nonatomic) IBOutlet UILabel *labelEmailError;
@property (weak, nonatomic) IBOutlet UITextField *textEmail;

@property (weak, nonatomic) IBOutlet UIButton *buttonOK;

@property (weak, nonatomic) IBOutlet UIView *viewPager;

@property (strong, nonatomic) NSString *name;
@property (strong, nonatomic) NSString *phone;
@property (strong, nonatomic) NSString *email;

@property (strong, nonatomic) User *user;

@property (assign, nonatomic) CGRect rect;

- (IBAction)onBack:(id)sender;

@end

@implementation MoveUserViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    if(self.exit) {
        self.imageBackground.hidden = YES;
    }
    self.viewTable.layer.cornerRadius = 5;
    self.user = [User getCurrent];
    if(self.user.name != nil) {
        self.textName.text = self.user.name;
        [self showLabel:self.labelNameHeader control:self.textName animated:NO];
    }
    if(self.user.phone != nil) {
        self.textPhone.text = self.user.phone;
        [self showLabel:self.labelPhoneHeader control:self.textPhone animated:NO];
    }
    if(self.user.email != nil) {
        self.textEmail.text = self.user.email;
        [self showLabel:self.labelEmailHeader control:self.textEmail animated:NO];
    }
    self.rect = CGRectZero;
    if(!self.exit) {
        [Utils setText:self.labelTitle text:@"accountform-heading"];
    }
    else {
        self.labelTitle.text = @"Edit account info";
    }
    [Utils setText:self.labelSubTitle text:@"accountform-description_text"];
    [Utils setText:self.labelNameHeader text:@"accountform-name"];
    [Utils setText:self.textName text:@"accountform-name"];
    [Utils setText:self.labelPhoneHeader text:@"accountform-phone"];
    [Utils setText:self.textPhone text:@"accountform-phone"];
    [Utils setText:self.labelEmailHeader text:@"accountform-email"];
    [Utils setText:self.textEmail text:@"accountform-email"];
    if(!self.exit) {
        [Utils setText:self.buttonOK text:@"accountform-button_submit"];
    }
    else {
        [self.buttonOK setTitle:@"Save changes" forState:UIControlStateNormal];
        self.viewPager.hidden = YES;
        NSLayoutConstraint *constraintHeight = nil;
        NSLayoutConstraint *constraintBottom = nil;
        for(NSLayoutConstraint *constraint in self.viewContainer.constraints) {
            if(constraint.firstAttribute == NSLayoutAttributeHeight) {
                constraintHeight = constraint;
            }
            else if(constraint.secondAttribute == NSLayoutAttributeBottom && constraint.secondItem == self.viewTable) {
                constraintBottom = constraint;
            }
        }
        constraintHeight.constant -= constraintBottom.constant - 10;
        constraintBottom.constant = 10;
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(onShowKeyboard:) name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(onHideKeyboard:) name:UIKeyboardWillHideNotification object:nil];
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillHideNotification object:nil];
}

- (void)setConstraints:(int)height duration:(float)duration {
    NSLayoutConstraint *constraintTop = nil;
    NSLayoutConstraint *constraintBottom = nil;
    NSLayoutConstraint *constraintHeight = nil;
    for(NSLayoutConstraint *constraint in self.scrollView.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeTop && constraint.firstItem == self.viewContainer) {
            constraintTop = constraint;
        }
        else if(constraint.secondAttribute == NSLayoutAttributeBottom && constraint.secondItem == self.viewContainer) {
            constraintBottom = constraint;
        }
    }
    for(NSLayoutConstraint *constraint in self.viewContainer.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeHeight) {
            constraintHeight = constraint;
        }
    }
    CGFloat top;
    CGFloat bottom;
    if(self.scrollView.frame.size.height - height - constraintHeight.constant - [self.topLayoutGuide length] > 0) {
        top = (self.scrollView.frame.size.height - height - constraintHeight.constant - [self.topLayoutGuide length]) / 2 + [self.topLayoutGuide length];
        bottom = self.scrollView.frame.size.height - constraintHeight.constant - top;
    }
    else {
        top = [self.topLayoutGuide length];
        bottom = height;
    }
    if(duration > 0) {
        [self.scrollView layoutIfNeeded];
        [UIView animateWithDuration:duration animations:^{
            constraintTop.constant = top;
            constraintBottom.constant = bottom;
            [self.scrollView layoutIfNeeded];
        }];
    }
    else {
        constraintTop.constant = top;
        constraintBottom.constant = bottom;
    }
}

- (void)viewDidLayoutSubviews {
    [super viewDidLayoutSubviews];
    if(!CGRectEqualToRect(self.rect, self.view.frame)) {
        [self setConstraints:0 duration:0];
        self.rect = self.view.frame;
    }
}

- (void)onShowKeyboard:(NSNotification *)notification {
    CGRect rect = ((NSValue *)[notification.userInfo objectForKey:UIKeyboardFrameEndUserInfoKey]).CGRectValue;
    double duration = ((NSNumber *)[notification.userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey]).doubleValue;
    [self setConstraints:rect.size.height duration:duration];
}

- (void)onHideKeyboard:(NSNotification *)notification {
    double duration = ((NSNumber *)[notification.userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey]).doubleValue;
    [self setConstraints:0 duration:duration];
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

- (IBAction)onBack:(id)sender {
    if([self.textName isFirstResponder]) {
        [self.textName resignFirstResponder];
    }
    if([self.textPhone isFirstResponder]) {
        [self.textPhone resignFirstResponder];
    }
    if([self.textEmail isFirstResponder]) {
        [self.textEmail resignFirstResponder];
    }
    [self dismissViewControllerAnimated:YES completion:^{
        
    }];
}

- (void)showLabel:(UILabel *)label control:(UIControl *)control animated:(BOOL)animated {
    if(animated) {
        CATransition *transition = [[CATransition alloc] init];
        transition.type = kCATransitionFade;
        transition.subtype = kCATransitionFromBottom;
        transition.duration = 0.25;
        [label.layer addAnimation:transition forKey:kCATransition];
    }
    label.hidden = NO;
    if([control isKindOfClass:[UIButton class]]) {
        [(UIButton *)control setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
    }
    for(NSLayoutConstraint *constraint in control.superview.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeTop && constraint.firstItem == control) {
            constraint.constant = 30;
        }
    }
}

- (void)hideLabel:(UILabel *)label control:(UIControl *)control animated:(BOOL)animated {
    if(animated) {
        CATransition *transition = [[CATransition alloc] init];
        transition.type = kCATransitionFade;
        transition.subtype = kCATransitionFromBottom;
        transition.duration = 0.25;
        [label.layer addAnimation:transition forKey:kCATransition];
    }
    label.hidden = YES;
    if([control isKindOfClass:[UIButton class]]) {
        [(UIButton *)control setTitleColor:[UIColor colorWithWhite:0.785f alpha:1] forState:UIControlStateNormal];
    }
    for(NSLayoutConstraint *constraint in control.superview.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeTop && constraint.firstItem == control) {
            constraint.constant = 5;
        }
    }
}

- (BOOL)check:(BOOL)valid label:(UILabel *)label {
    CATransition *transition = [[CATransition alloc] init];
    transition.type = kCATransitionFade;
    transition.subtype = kCATransitionFromBottom;
    transition.duration = 0.25;
    [label.layer addAnimation:transition forKey:kCATransition];
    if(valid) {
        label.hidden = YES;
        return YES;
    }
    else {
        label.hidden = NO;
        return NO;
    }
}

- (IBAction)onOK:(id)sender {
    BOOL checkName = [self check:[Utils validateName:self.textName.text] label:self.labelNameError];
    BOOL checkPhone = [self check:[Utils validatePhone:self.textPhone.text] label:self.labelPhoneError];
    BOOL checkEmail = [self check:[Utils validateEmail:self.textEmail.text] label:self.labelEmailError];
    if(checkName && checkPhone && checkEmail) {
        self.user.name = self.textName.text;
        self.user.phone = self.textPhone.text;
        self.user.email = self.textEmail.text;
        NSString *from = [NSString stringWithFormat:@"%@, %@", (NSString *)[self.user.zip objectForKey:@"city"], (NSString *)[self.user.zip objectForKey:@"state"]];
        NSString *to = [NSString stringWithFormat:@"%@, %@", (NSString *)[self.user.city objectForKey:@"name"], (NSString *)[self.user.state objectForKey:@"name"]];
        [URLRequests getDistanceFrom:from to:to receive:^(NSString *distance) {
            self.user.distance = distance;
            [self.user save];
            if([self.textName isFirstResponder]) {
                [self.textName resignFirstResponder];
            }
            if([self.textPhone isFirstResponder]) {
                [self.textPhone resignFirstResponder];
            }
            if([self.textEmail isFirstResponder]) {
                [self.textEmail resignFirstResponder];
            }
            if(self.exit) {
                [self dismissViewControllerAnimated:YES completion:^{
                    if(self.close != nil) {
                        self.close();
                    }
                }];
            }
            else {
                [self.presentingViewController.presentingViewController.presentingViewController dismissViewControllerAnimated:YES completion:^{

                }];
            }
        }];
    }
}

#pragma mark - UITextFieldDelegate

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string {
    NSString *text = [textField.text stringByReplacingCharactersInRange:range withString:string];
    if(textField == self.textName) {
        if(![text isEqualToString:@""]) {
            [self showLabel:self.labelNameHeader control:self.textName animated:YES];
        }
        else {
            [self hideLabel:self.labelNameHeader control:self.textName animated:YES];
        }
        [self check:YES label:self.labelNameError];
    }
    else if(textField == self.textPhone) {
        if(![text isEqualToString:@""]) {
            [self showLabel:self.labelPhoneHeader control:self.textPhone animated:YES];
        }
        else {
            [self hideLabel:self.labelPhoneHeader control:self.textPhone animated:YES];
        }
        [self check:YES label:self.labelPhoneError];
    }
    else if(textField == self.textEmail) {
        if(![text isEqualToString:@""]) {
            [self showLabel:self.labelEmailHeader control:self.textEmail animated:YES];
        }
        else {
            [self hideLabel:self.labelEmailHeader control:self.textEmail animated:YES];
        }
        [self check:YES label:self.labelEmailError];
    }
    return YES;
}

- (void)textFieldDidBeginEditing:(UITextField *)textField {
}

- (void)textFieldDidEndEditing:(UITextField *)textField {
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    [textField resignFirstResponder];
    return YES;
}

@end
