//
//  MoverInfoTableViewCell.m
//  MoveAdvisor
//
//  Created by Outsourcing IT Services Ltd. on 7/4/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "MoverInfoTableViewCell.h"
#import "MoverNumberCollectionViewCell.h"
#import "MoverRatingTableViewCell.h"

@interface MoverInfoTableViewCell ()

@property (weak, nonatomic) IBOutlet UILabel *labelInfo;
@property (weak, nonatomic) IBOutlet UICollectionView *collectionNumbers;
@property (weak, nonatomic) IBOutlet UILabel *labelTotalReviews;
@property (weak, nonatomic) IBOutlet UIImageView *imageRating;
@property (weak, nonatomic) IBOutlet UITableView *tableRatings;

@property (strong, nonatomic) NSMutableArray<NSDictionary<NSString *, NSString *> *> *numbers;
@property (strong, nonatomic) NSMutableArray<NSDictionary<NSString *, NSObject *> *> *ratings;

@property (assign, nonatomic) NSUInteger reviews;

@end

@implementation MoverInfoTableViewCell

- (void)awakeFromNib {
    [super awakeFromNib];
    // Initialization code
}

- (void)setSelected:(BOOL)selected animated:(BOOL)animated {
    [super setSelected:selected animated:animated];

    // Configure the view for the selected state
}

- (void)setData:(NSDictionary *)data reviews:(NSUInteger)reviews {
    self.labelInfo.attributedText = (NSAttributedString *)[data objectForKey:@"info"];
    self.numbers = (NSMutableArray<NSDictionary<NSString *, NSString *> *> *)[data objectForKey:@"numbers"];
    [self.collectionNumbers reloadData];
    self.reviews = reviews;
    self.labelTotalReviews.text = [NSString stringWithFormat:@"Total %lu moving reviews", (unsigned long)reviews];
    self.ratings = (NSMutableArray<NSDictionary<NSString *, NSObject *> *> *)[data objectForKey:@"ratings"];
    self.imageRating.image = [UIImage imageNamed:[NSString stringWithFormat:@"rating%@", (NSString *)[data objectForKey:@"rating"]]];
    [self.tableRatings reloadData];
}

#pragma mark - UICollectionViewDataSource

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return self.numbers.count;
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    MoverNumberCollectionViewCell *cell = (MoverNumberCollectionViewCell *)[collectionView dequeueReusableCellWithReuseIdentifier:@"cell" forIndexPath:indexPath];
    [cell setData:[self.numbers objectAtIndex:indexPath.row]];
    return cell;
}

#pragma mark - UICollectionViewDelegate

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
}

#pragma mark - UICollectionViewDelegateFlowLayout

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath {
    NSString *text = [[self.numbers objectAtIndex:indexPath.row] objectForKey:@"text"];
    CGSize size = [text sizeWithAttributes:@{NSFontAttributeName: [UIFont systemFontOfSize:17]}];
    size.width = ceilf(size.width) + 80;
    size.height = ((UICollectionViewFlowLayout *)collectionViewLayout).itemSize.height;
    return size;
}


#pragma mark - UITableViewDataSource

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.ratings.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    MoverRatingTableViewCell *cell = (MoverRatingTableViewCell *)[tableView dequeueReusableCellWithIdentifier:@"cell"];
    [cell setData:[self.ratings objectAtIndex:indexPath.row] reviews:self.reviews size:tableView.frame.size.width];
    return cell;
}

#pragma mark - UITableViewDelegate


@end
