//
//  MoversViewController.m
//  MoveAdvisor
//
//  Created by Outsourcing IT Services Ltd. on 6/27/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "MoversViewController.h"
#import "MoversTableViewCell.h"
#import "MoverViewController.h"
#import "User.h"
#import "Utils.h"

@interface MoversViewController ()

@property (weak, nonatomic) IBOutlet UILabel *labelTitle;
@property (weak, nonatomic) IBOutlet UILabel *labelMovers;
@property (weak, nonatomic) IBOutlet UITableView *tableMovers;
@property (weak, nonatomic) IBOutlet UIActivityIndicatorView *progressIndicator;

@property (strong, nonatomic) NSArray *moversRating;
@property (strong, nonatomic) NSArray *moversReviews;
@property (strong, nonatomic) NSArray *movers;

- (IBAction)onBack:(id)sender;
- (IBAction)onSort:(id)sender;

@end

@implementation MoversViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    User *user = [User getCurrent];
    self.labelTitle.text = [NSString stringWithFormat:@"%@ Shippers", [user.zip objectForKey:@"city"]];
    NSString *path = [[NSBundle mainBundle] pathForResource:@"states" ofType:@"json"];
    NSData *data = [NSData dataWithContentsOfFile:path];
    NSDictionary *state = nil;
    if(data != nil) {
        NSError *jsonerror;
        NSDictionary<NSString *, NSDictionary *> *json = (NSDictionary<NSString *, NSDictionary *> *)[NSJSONSerialization JSONObjectWithData:data options:0 error:&jsonerror];
        if(json != nil) {
            state = [json objectForKey:[user.zip objectForKey:@"state"]];
        }
    }
    NSURLSessionConfiguration *configuration = [NSURLSessionConfiguration defaultSessionConfiguration];
    NSURLSession *session = [NSURLSession sessionWithConfiguration:configuration delegate:nil delegateQueue:[NSOperationQueue mainQueue]];
    NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:[NSString stringWithFormat:@"https://www.mymovingreviews.com/iphoneapprequest.php?get=moversbystate&stateID=%d&output=json&ctype=car-moving", ((NSString *)[state objectForKey:@"id"]).intValue]]];
    NSURLSessionDataTask *task = [session dataTaskWithRequest:request completionHandler:^(NSData *data, NSURLResponse *response, NSError *error) {
        if(data != nil) {
            NSError *jsonerror;
            self.movers = [NSJSONSerialization JSONObjectWithData:data options:0 error:&jsonerror];
            self.labelMovers.text = [NSString stringWithFormat:@"Total %lu shipping companies", (unsigned long)self.movers.count];
            self.moversRating = self.movers;
            self.moversReviews = [self.movers sortedArrayUsingComparator:^NSComparisonResult(NSDictionary *obj1, NSDictionary *obj2) {
                int reviews1 = ((NSString *)[obj1 objectForKey:@"total_reviews"]).intValue;
                int reviews2 = ((NSString *)[obj2 objectForKey:@"total_reviews"]).intValue;
                if(reviews1 > reviews2) {
                    return -1;
                }
                else if(reviews1 < reviews2) {
                    return 1;
                }
                else {
                    return 0;
                }
            }];
            [self.tableMovers reloadData];
            [self.progressIndicator stopAnimating];
        }
    }];
    [self.progressIndicator startAnimating];
    [task resume];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
    if([segue.destinationViewController isKindOfClass:[MoverViewController class]]) {
        ((MoverViewController *)segue.destinationViewController).moverid = (NSString *)[(NSDictionary *)[self.movers objectAtIndex:[self.tableMovers indexPathForCell:(MoversTableViewCell *)sender].row] objectForKey:@"id"];
    }
}


- (UIImage *)loadImage:(NSString *)url {
    return [Utils loadImage:url download:^(UIImage *image) {
        for(NSIndexPath *path in self.tableMovers.indexPathsForVisibleRows) {
            NSDictionary *mover = (NSDictionary *)[self.movers objectAtIndex:path.row];
            NSString *logo = (NSString *)[mover objectForKey:@"logo_url"];
            if(logo != nil && [logo isEqualToString:url]) {
                MoversTableViewCell *cell = (MoversTableViewCell *)[self.tableMovers cellForRowAtIndexPath:path];
                if(cell != nil) {
                    [cell setImage:image background:[UIColor clearColor]];
                }
            }
        }
    }];
}

- (IBAction)onBack:(id)sender {
    [self dismissViewControllerAnimated:YES completion:^{
        
    }];
}

- (IBAction)onSort:(id)sender {
    switch(((UISegmentedControl *)sender).selectedSegmentIndex) {
        case 0:
            self.movers = self.moversRating;
            break;
        case 1:
            self.movers = self.moversReviews;
            break;
    }
    [self.tableMovers reloadData];
}

#pragma mark - UITableViewDataSource

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.movers.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    MoversTableViewCell *cell = (MoversTableViewCell *)[tableView dequeueReusableCellWithIdentifier:@"cell"];
    NSDictionary *mover = (NSDictionary *)[self.movers objectAtIndex:indexPath.row];
    NSString *logo = (NSString *)[mover objectForKey:@"logo_url"];
    UIImage *image;
    UIColor *background;
    if(logo != nil && ![logo isEqualToString:@""]) {
        image = [self loadImage:logo];
        background = [UIColor clearColor];
    }
    else {
        image = [UIImage imageNamed:@"no_logo"];
        background = [Utils getColor:0xFFFAFAFA];
    }
    [cell setImage:image background:background];
    [cell setData:mover];
    return cell;
}

#pragma mark - UITableViewDelegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {

}

@end
