//
//  QuoteViewController.m
//  MoveAdvisor
//
//  Created by Outsourcing IT Services Ltd. on 9/30/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "QuoteViewController.h"
#import "MoveViewController.h"
#import "MoveRideViewController.h"
#import "MoveUserViewController.h"
#import "QuoteCollectionReusableView.h"
#import "QuoteCollectionViewCell.h"
#import "QuoteButtonCollectionViewCell.h"
#import "Utils.h"
#import "User.h"
#import "URLRequests.h"

@interface QuoteViewController ()

@property (weak, nonatomic) IBOutlet UILabel *labelTitle;

@property (weak, nonatomic) IBOutlet UICollectionView *collectionQuote;

@property (weak, nonatomic) IBOutlet UIButton *buttonQuote;

@property (strong, nonatomic) User *user;

@property (strong, nonatomic) NSString *move;
@property (strong, nonatomic) NSString *ride;
@property (strong, nonatomic) NSString *account;
@property (strong, nonatomic) NSString *month;
@property (strong, nonatomic) NSString *day;

- (IBAction)onBack:(id)sender;

@end

@implementation QuoteViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    [Utils setText:self.labelTitle text:@"getquotescreen-title"];
    [Utils setText:self.buttonQuote text:@"getquotescreen-button_submit"];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self load];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)load {
    self.user = [User getCurrent];
    NSDictionary<NSString *, NSNumber *> *date = self.user.date;
    NSCalendar *calendar = [NSCalendar calendarWithIdentifier:NSCalendarIdentifierGregorian];
    calendar.locale = [NSLocale localeWithLocaleIdentifier:@"en_us"];
    self.month = [calendar.monthSymbols objectAtIndex:[date objectForKey:@"month"].intValue - 1];
    self.day = [date objectForKey:@"day"].stringValue;
    NSString *from = [NSString stringWithFormat:@"%@, %@", (NSString *)[self.user.zip objectForKey:@"city"], (NSString *)[self.user.zip objectForKey:@"state"]];
    NSString *to = [NSString stringWithFormat:@"%@, %@", (NSString *)[self.user.city objectForKey:@"name"], (NSString *)[self.user.state objectForKey:@"name"]];
    NSString *distance = self.user.distance;
    self.move = [NSString stringWithFormat:@"Move from: %@\nMove to: %@\nDistance: %@", from, to, distance];
    self.ride = [NSString stringWithFormat:@"Ride make: %@\nRide model: %@\nRide condition: %@\nShipping method: %@", (NSString *)[self.user.make objectForKey:@"make_name"], (NSString *)[self.user.model objectForKey:@"model_name"], (NSString *)[self.user.condition objectForKey:@"name"], (NSString *)[self.user.method objectForKey:@"name"]];
    self.account = [NSString stringWithFormat:@"Name: %@\nPhone: %@\nEmail: %@", self.user.name, self.user.phone, self.user.email];
    [self.collectionQuote reloadData];
}

#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
    if([segue.destinationViewController isKindOfClass:[MoveViewController class]]) {
        //((MoveViewController *)segue.destinationViewController).user = [User getCurrent];
        ((MoveViewController *)segue.destinationViewController).exit = YES;
        ((MoveViewController *)segue.destinationViewController).close = ^(NSDictionary *size) {
            [self load];
        };
    }
    else if([segue.destinationViewController isKindOfClass:[MoveRideViewController class]]) {
        ((MoveRideViewController *)segue.destinationViewController).exit = YES;
        ((MoveRideViewController *)segue.destinationViewController).close = ^(NSDictionary *size) {
            [self load];
        };
    }
    else if([segue.destinationViewController isKindOfClass:[MoveUserViewController class]]) {
        ((MoveUserViewController *)segue.destinationViewController).exit = YES;
        ((MoveUserViewController *)segue.destinationViewController).close = ^(NSDictionary *size) {
            [self load];
        };
    }
}


- (IBAction)onBack:(id)sender {
    [self dismissViewControllerAnimated:YES completion:^{
        
    }];
}

#pragma mark - UICollectionViewDataSource

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView {
    return 4;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return 1;
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    if(indexPath.section < 3) {
        QuoteCollectionViewCell *cell = (QuoteCollectionViewCell *)[collectionView dequeueReusableCellWithReuseIdentifier:@"cell" forIndexPath:indexPath];
        switch(indexPath.section) {
            case 0:
                [cell setText:self.move month:self.month day:self.day /*edit:move*/];
                break;
            case 1:
                [cell setText:self.ride image:@"ride-model" /*edit:size*/];
                break;
            case 2:
                [cell setText:self.account image:@"user-card" /*edit:user*/];
                break;
        }
        return cell;
    }
    else {
        QuoteButtonCollectionViewCell *cell = (QuoteButtonCollectionViewCell *)[collectionView dequeueReusableCellWithReuseIdentifier:@"quote" forIndexPath:indexPath];
        return cell;
    }
}

- (UICollectionReusableView *)collectionView:(UICollectionView *)collectionView viewForSupplementaryElementOfKind:(NSString *)kind atIndexPath:(NSIndexPath *)indexPath {
    QuoteCollectionReusableView *cell = (QuoteCollectionReusableView *)[collectionView dequeueReusableSupplementaryViewOfKind:kind withReuseIdentifier:@"group" forIndexPath:indexPath];
    if(indexPath.section < 3) {
        NSString *text;
        switch(indexPath.section) {
            case 0:
                text = [Utils getText:@"getquotescreen-title_shipping_data"];
                if(text == nil) {
                    text = @"Shipping Details";
                }
                [cell setText:text];
                break;
            case 1:
                text = [Utils getText:@"getquotescreen-title_ride_data"];
                if(text == nil) {
                    text = @"Ride Details";
                }
                [cell setText:text];
                break;
            case 2:
                text = [Utils getText:@"getquotescreen-title_user_data"];
                if(text == nil) {
                    text = @"Account Details";
                }
                [cell setText:text];
                break;
        }
    }
    else {
        [cell setText:nil];
    }
    return cell;
}

#pragma mark - UICollectionViewDelegate

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    if(indexPath.section < 3) {
        switch(indexPath.section) {
            case 0:
                [self performSegueWithIdentifier:@"move" sender:self];
                break;
            case 1:
                [self performSegueWithIdentifier:@"ride" sender:self];
                break;
            case 2:
                [self performSegueWithIdentifier:@"user" sender:self];
                break;
        }
    }
    else {
        UIViewController *controller = [[UIViewController alloc] init];
        controller.modalPresentationStyle = UIModalPresentationOverFullScreen;
        controller.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
        UIView *view = [[UIView alloc] init];
        view.backgroundColor = [Utils getColor:0x80000000];
        controller.view = view;
        UIActivityIndicatorView *progress = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhiteLarge];
        progress.color = [Utils getColor:COLOR_GREEN];
        progress.translatesAutoresizingMaskIntoConstraints = NO;
        [progress startAnimating];
        [view addSubview:progress];
        [view addConstraint:[NSLayoutConstraint constraintWithItem:progress attribute:NSLayoutAttributeCenterX relatedBy:NSLayoutRelationEqual toItem:view attribute:NSLayoutAttributeCenterX multiplier:1 constant:0]];
        [view addConstraint:[NSLayoutConstraint constraintWithItem:progress attribute:NSLayoutAttributeCenterY relatedBy:NSLayoutRelationEqual toItem:view attribute:NSLayoutAttributeCenterY multiplier:1 constant:0]];
        [self presentViewController:controller animated:YES completion:^{
            [URLRequests sendQuote:^(BOOL success, NSString *error){
                [self dismissViewControllerAnimated:YES completion:^{
                    if(success) {
                        [self performSegueWithIdentifier:@"sent" sender:self];
                    }
                    else {
                        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Failed to send quote!" message:error delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
                        [alert show];
                    }
                }];
            }];
        }];
    }
}

#pragma mark - UICollectionViewDelegateFlowLayout

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath {
    if(indexPath.section < 3) {
        CGSize size = CGSizeMake(collectionView.frame.size.width, ((UICollectionViewFlowLayout *)collectionViewLayout).itemSize.height);
        return size;
    }
    else {
        CGSize size = CGSizeMake(collectionView.frame.size.width, 70);
        return size;
    }
}

@end
