//
//  RatingLowerViewController.m
//  MoveAdvisor
//
//  Created by Outsourcing It Services Ltd on 3/7/17.
//  Copyright © 2017 Outsourcing IT Services Ltd. All rights reserved.
//

#import "RatingLowerViewController.h"
#import "Utils.h"
#import "User.h"
#import "URLRequests.h"

@interface RatingLowerViewController ()

@property (weak, nonatomic) IBOutlet UIScrollView *scrollView;
@property (weak, nonatomic) IBOutlet UIView *viewContainer;
@property (weak, nonatomic) IBOutlet UITextView *textView;

@property (weak, nonatomic) IBOutlet UIImageView *imageRating1;
@property (weak, nonatomic) IBOutlet UIImageView *imageRating2;
@property (weak, nonatomic) IBOutlet UIImageView *imageRating3;
@property (weak, nonatomic) IBOutlet UIImageView *imageRating4;
@property (weak, nonatomic) IBOutlet UIImageView *imageRating5;

@property (assign, nonatomic) BOOL hasText;

@property (assign, nonatomic) CGRect rect;

- (IBAction)onRating:(id)sender;
- (IBAction)onClose:(id)sender;

@end

@implementation RatingLowerViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.viewContainer.layer.cornerRadius = 5;
    if(self.rating >= 1) {
        self.imageRating1.image = [UIImage imageNamed:@"star-selected"];
    }
    if(self.rating >= 2) {
        self.imageRating2.image = [UIImage imageNamed:@"star-selected"];
    }
    if(self.rating >= 3) {
        self.imageRating3.image = [UIImage imageNamed:@"star-selected"];
    }
    if(self.rating >= 4) {
        self.imageRating4.image = [UIImage imageNamed:@"star-selected"];
    }
    if(self.rating >= 5) {
        self.imageRating5.image = [UIImage imageNamed:@"star-selected"];
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(onShowKeyboard:) name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(onHideKeyboard:) name:UIKeyboardWillHideNotification object:nil];
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillHideNotification object:nil];
}

- (void)setConstraints:(int)height duration:(float)duration {
    NSLayoutConstraint *constraintTop = nil;
    NSLayoutConstraint *constraintBottom = nil;
    NSLayoutConstraint *constraintHeight = nil;
    for(NSLayoutConstraint *constraint in self.scrollView.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeTop && constraint.firstItem == self.viewContainer) {
            constraintTop = constraint;
        }
        else if(constraint.secondAttribute == NSLayoutAttributeBottom && constraint.secondItem == self.viewContainer) {
            constraintBottom = constraint;
        }
    }
    for(NSLayoutConstraint *constraint in self.viewContainer.constraints) {
        if(constraint.firstAttribute == NSLayoutAttributeHeight) {
            constraintHeight = constraint;
        }
    }
    CGFloat top;
    CGFloat bottom;
    if(self.scrollView.frame.size.height - height - constraintHeight.constant /*- [self.topLayoutGuide length]*/ > 0) {
        top = (self.scrollView.frame.size.height - height - constraintHeight.constant /*- [self.topLayoutGuide length]*/) / 2; // + [self.topLayoutGuide length];
        bottom = self.scrollView.frame.size.height - constraintHeight.constant - top;
    }
    else {
        top = 0; //[self.topLayoutGuide length];
        bottom = height;
    }
    if(duration > 0) {
        [self.scrollView layoutIfNeeded];
        [UIView animateWithDuration:duration animations:^{
            constraintTop.constant = top;
            constraintBottom.constant = bottom;
            [self.scrollView layoutIfNeeded];
        }];
    }
    else {
        constraintTop.constant = top;
        constraintBottom.constant = bottom;
    }
}

- (void)viewDidLayoutSubviews {
    [super viewDidLayoutSubviews];
    if(!CGRectEqualToRect(self.rect, self.view.frame)) {
        [self setConstraints:0 duration:0];
        self.rect = self.view.frame;
    }
}

- (void)onShowKeyboard:(NSNotification *)notification {
    CGRect rect = ((NSValue *)[notification.userInfo objectForKey:UIKeyboardFrameEndUserInfoKey]).CGRectValue;
    double duration = ((NSNumber *)[notification.userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey]).doubleValue;
    [self setConstraints:rect.size.height duration:duration];
}

- (void)onHideKeyboard:(NSNotification *)notification {
    double duration = ((NSNumber *)[notification.userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey]).doubleValue;
    [self setConstraints:0 duration:duration];
}

/*
 #pragma mark - Navigation
 
 // In a storyboard-based application, you will often want to do a little preparation before navigation
 - (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
 // Get the new view controller using [segue destinationViewController].
 // Pass the selected object to the new view controller.
 }
 */

- (IBAction)onRating:(id)sender {
    NSDictionary<NSString *, NSString *> *data = @{
                                                   @"name": [NSString stringWithFormat:@"%d stars", self.rating],
                                                   @"email": @"feedback@rideshipper.com",
                                                   @"message": self.textView.text
                                                   };
    UIViewController *controller = [[UIViewController alloc] init];
    controller.modalPresentationStyle = UIModalPresentationOverFullScreen;
    controller.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
    UIView *view = [[UIView alloc] init];
    view.backgroundColor = [Utils getColor:0x80000000];
    controller.view = view;
    UIActivityIndicatorView *progress = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhiteLarge];
    progress.color = [Utils getColor:COLOR_GREEN];
    progress.translatesAutoresizingMaskIntoConstraints = NO;
    [progress startAnimating];
    [view addSubview:progress];
    [view addConstraint:[NSLayoutConstraint constraintWithItem:progress attribute:NSLayoutAttributeCenterX relatedBy:NSLayoutRelationEqual toItem:view attribute:NSLayoutAttributeCenterX multiplier:1 constant:0]];
    [view addConstraint:[NSLayoutConstraint constraintWithItem:progress attribute:NSLayoutAttributeCenterY relatedBy:NSLayoutRelationEqual toItem:view attribute:NSLayoutAttributeCenterY multiplier:1 constant:0]];
    [self presentViewController:controller animated:YES completion:^{
        [URLRequests sendEmail:data to:@"about" receive:^(BOOL success, NSString *error){
            [self dismissViewControllerAnimated:YES completion:^{
                if(success) {
                    User *user = [User getCurrent];
                    user.rated = YES;
                    [user save];
                    [self.presentingViewController.presentingViewController dismissViewControllerAnimated:YES completion:^{
                        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Message successfully sent!" message:@"Thank you! We appreciate your effort to provide us with feedback!" delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
                        [alert show];
                    }];
                }
                else {
                    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Failed to send message!" message:error delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
                    [alert show];
                }
            }];
        }];
    }];
}

- (IBAction)onClose:(id)sender {
    User *user = [User getCurrent];
    user.launchcount = 0;
    user.launchdate = [NSDate date];
    [user save];
    [self.presentingViewController.presentingViewController dismissViewControllerAnimated:YES completion:^{
        
    }];
}

#pragma mark - UITextViewDelegate

- (BOOL)textView:(UITextView *)textView shouldChangeTextInRange:(NSRange)range replacementText:(NSString *)text {
    if([text isEqualToString:@""]) {
        if(self.hasText) {
            if([[self.textView.text stringByReplacingCharactersInRange:range withString:text] isEqualToString:@""]) {
                self.hasText = NO;
                self.textView.text = @"Review (Optional) ";
                self.textView.textColor = [UIColor lightGrayColor];
                return NO;
            }
            else {
                return YES;
            }
        }
        else {
            return NO;
        }
    }
    else {
        if(!self.hasText) {
            self.hasText = YES;
            self.textView.text = text;
            self.textView.textColor = [UIColor blackColor];
            return NO;
        }
        else {
            return YES;
        }
    }
}

@end
