//
//  URLRequests.m
//  MoveAdvisor
//
//  Created by Outsourcing IT Services Ltd. on 9/30/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "URLRequests.h"
#import "User.h"
#import "Utils.h"
#import "Text.h"

static NSString * const RequestTypeGet = @"GET";
static NSString * const RequestTypePost = @"POST";

static int const ResponseOK = 200;

@implementation URLRequests

+ (NSString *)encode:(NSString *)param {
    NSMutableCharacterSet *set = (NSMutableCharacterSet *)[[NSMutableCharacterSet URLQueryAllowedCharacterSet] mutableCopy];
    [set removeCharactersInString:@"&="];
    return [param stringByAddingPercentEncodingWithAllowedCharacters:set];
}

+ (void)sendRequest:(NSString *)url type:(NSString *)type params:(NSDictionary *)params receive:(void(^)(NSData *data))receive {
    NSURLSessionConfiguration *configuration = [NSURLSessionConfiguration defaultSessionConfiguration];
    NSURLSession *session = [NSURLSession sessionWithConfiguration:configuration delegate:nil delegateQueue:[NSOperationQueue mainQueue]];
    NSMutableURLRequest *request = [[NSMutableURLRequest alloc] init];
    request.HTTPMethod = type;
    NSMutableString *query = nil;
    if(params != nil && params.count > 0) {
        query = [[NSMutableString alloc] init];
        for(NSObject *key in params) {
            if(query.length > 0) {
                [query appendString:@"&"];
            }
            id param = [params objectForKey:key];
            if([param isKindOfClass:[NSString class]]) {
                [query appendFormat:@"%@=%@", key, [self encode:param]];
            }
            else if([param isKindOfClass:[NSNumber class]]) {
                [query appendFormat:@"%@=%@", key, param];
            }
            else if([param isKindOfClass:[NSArray class]]) {
                NSError *error;
                NSString *paramstring = [[NSString alloc] initWithData:[NSJSONSerialization dataWithJSONObject:param options:0 error:&error] encoding:NSUTF8StringEncoding];
                [query appendFormat:@"%@=%@", key, [self encode:paramstring]];
            }
            else if([param isKindOfClass:[NSDictionary class]]) {
                NSError *error;
                NSString *paramstring = [[NSString alloc] initWithData:[NSJSONSerialization dataWithJSONObject:param options:0 error:&error] encoding:NSUTF8StringEncoding];
                [query appendFormat:@"%@=%@", key, [self encode:paramstring]];
            }
        }
    }
    if(query != nil) {
        if([type isEqualToString:RequestTypeGet]) {
            url = [NSString stringWithFormat:@"%@?%@", url, query];
        }
        if([type isEqualToString:RequestTypePost]) {
            request.HTTPBody = [query dataUsingEncoding:NSUTF8StringEncoding];
        }
    }
    request.URL = [NSURL URLWithString:url];
    NSURLSessionDataTask *task = [session dataTaskWithRequest:request completionHandler:^(NSData *data, NSURLResponse *response, NSError *error) {
        if(response != nil && ((NSHTTPURLResponse *)response).statusCode == ResponseOK) {
            receive(data);
        }
        else {
            receive(nil);
        }
    }];
    [task resume];
}

+ (void)getMakes:(void (^)(NSArray *makes))receive {
    NSString *url = @"https://www.mymovingloads.com/services/getmakes?site_id=2";
    [URLRequests sendRequest:url type:RequestTypeGet params:nil receive:^(NSData *data) {
        NSArray *makes = nil;
        if(data != nil) {
            NSError *jsonerror;
            id obj = [NSJSONSerialization JSONObjectWithData:data options:0 error:&jsonerror];
            if(obj != nil && [obj isKindOfClass:[NSArray class]]) {
                makes = (NSArray *)obj;
            }
        }
        if(receive != nil) {
            receive(makes);
        }
    }];
}

+ (void)getModels:(NSString *)make receive:(void (^)(NSArray *models))receive {
    NSString *url = [NSString stringWithFormat:@"https://www.mymovingloads.com/services/getmodels?site_id=2&make_name=%@", [self encode:make]];
    [URLRequests sendRequest:url type:RequestTypeGet params:nil receive:^(NSData *data) {
        NSArray *models = nil;
        if(data != nil) {
            NSError *jsonerror;
            id obj = [NSJSONSerialization JSONObjectWithData:data options:0 error:&jsonerror];
            if(obj != nil && [obj isKindOfClass:[NSArray class]]) {
                models = (NSArray *)obj;
            }
        }
        if(receive != nil) {
            receive(models);
        }
    }];
}

+ (void)getDistanceFrom:(NSString *)from to:(NSString *)to receive:(void (^)(NSString *distance))receive {
    NSString *url = [NSString stringWithFormat:@"https://maps.googleapis.com/maps/api/distancematrix/json?units=imperial&origins=%@&destinations=%@", [self encode:from], [self encode:to]];
    [URLRequests sendRequest:url type:RequestTypeGet params:nil receive:^(NSData *data) {
        NSString *distance = nil;
        if(data != nil) {
            NSError *jsonerror;
            id obj = [NSJSONSerialization JSONObjectWithData:data options:0 error:&jsonerror];
            if(obj != nil && [obj isKindOfClass:[NSDictionary class]]) {
                NSString *status = (NSString *)[obj objectForKey:@"status"];
                if([status isEqualToString:@"OK"]) {
                    NSArray *rows = (NSArray *)[obj objectForKey:@"rows"];
                    if(rows.count > 0) {
                        NSArray *elements = (NSArray *)[(NSDictionary *)[rows objectAtIndex:0] objectForKey:@"elements"];
                        if(elements.count > 0) {
                            NSDictionary *element = (NSDictionary *)[elements objectAtIndex:0];
                            status = (NSString *)[obj objectForKey:@"status"];
                            if([status isEqualToString:@"OK"]) {
                                distance = (NSString *)[(NSDictionary *)[(NSDictionary *)element objectForKey:@"distance"] objectForKey:@"text"];
                            }
                        }
                    }
                }
            }
        }
        if(receive != nil) {
            receive(distance);
        }
    }];
}

+ (void)sendQuoteEmail:(NSString *)message emails:(NSArray *)emails inventory:(BOOL)inventory receive:(void (^)(BOOL success, NSString *error))receive {
    User *user = [User getCurrent];
    
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    formatter.locale = [NSLocale localeWithLocaleIdentifier:@"en_US"];
    formatter.dateFormat = @"yyyy-MM-dd";
    NSCalendar *calendar = [NSCalendar calendarWithIdentifier:NSCalendarIdentifierGregorian];
    NSDateComponents *components = [[NSDateComponents alloc] init];
    components.year = ((NSNumber *)[user.date objectForKey:@"year"]).intValue;
    components.month = ((NSNumber *)[user.date objectForKey:@"month"]).intValue;
    components.day = ((NSNumber *)[user.date objectForKey:@"day"]).intValue;
    components.timeZone = calendar.timeZone;
    NSString *date = [formatter stringFromDate:[calendar dateFromComponents:components]];
    
    NSMutableDictionary *content = [[NSMutableDictionary alloc] init];
    
    [content setObject:@0 forKey:@"email_context"];
    [content setObject:user.email forKey:@"from_email"];
    [content setObject:user.name forKey:@"from_name"];
    [content setObject:user.phone forKey:@"phone"];
    [content setObject:emails forKey:@"to"];
    [content setObject:@"Not set" forKey:@"to_company_name"];
    [content setObject:@"" forKey:@"subject"];
    [content setObject:message forKey:@"message"];
    
    [content setObject:[user.zip objectForKey:@"zip"] forKey:@"loc_from_zip"];
    [content setObject:[user.zip objectForKey:@"city"] forKey:@"loc_from_city"];
    [content setObject:[user.zip objectForKey:@"state"] forKey:@"loc_from_state"];
    [content setObject:[user.city objectForKey:@"name"] forKey:@"loc_to_city"];
    [content setObject:[user.state objectForKey:@"name"] forKey:@"loc_to_state"];
    [content setObject:user.distance forKey:@"move_distance"];
    
    [content setObject:date forKey:@"move_date_iso"];
    [content setObject:[user.date objectForKey:@"year"] forKey:@"move_year"];
    [content setObject:[user.date objectForKey:@"month"] forKey:@"move_month"];
    [content setObject:[user.date objectForKey:@"day"] forKey:@"move_date"];
    
    if(inventory) {
        [content setObject:[user.make objectForKey:@"make_name"] forKey:@"make"];
        [content setObject:[user.model objectForKey:@"model_name"] forKey:@"model"];
        [content setObject:[user.condition objectForKey:@"name"] forKey:@"vehicle_condition"];
        [content setObject:[user.method objectForKey:@"name"] forKey:@"shipping"];
    }
    
    NSString *url = @"https://www.mymovingloads.com/services/emails/inventory-email";
    NSDictionary *params = @{@"data": content};
    [URLRequests sendRequest:url type:RequestTypePost params:params receive:^(NSData *data) {
        NSError *jsonerror;
        id obj = [NSJSONSerialization JSONObjectWithData:data options:0 error:&jsonerror];
        if(obj != nil && [obj isKindOfClass:[NSDictionary class]]) {
            NSString *status = (NSString *)[obj objectForKey:@"status"];
            BOOL success;
            NSMutableString *errors;
            if(![status isEqualToString:@"error"]) {
                success = YES;
            }
            else {
                success = NO;
                errors = [[NSMutableString alloc] init];
                for(NSString *error in (NSArray *)[obj objectForKey:@"errors"]) {
                    if(errors.length > 0) {
                        [errors appendString:@"\n"];
                    }
                    [errors appendString:error];
                }
            }
            if(receive != nil) {
                receive(success, errors);
            }
        }
    }];
}

+ (void)sendQuote:(void (^)(BOOL success, NSString *error))receive {
    User *user = [User getCurrent];
    
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    formatter.locale = [NSLocale localeWithLocaleIdentifier:@"en_US"];
    formatter.dateFormat = @"yyyy-MM-dd";
    NSCalendar *calendar = [NSCalendar calendarWithIdentifier:NSCalendarIdentifierGregorian];
    NSDateComponents *components = [[NSDateComponents alloc] init];
    components.year = ((NSNumber *)[user.date objectForKey:@"year"]).intValue;
    components.month = ((NSNumber *)[user.date objectForKey:@"month"]).intValue;
    components.day = ((NSNumber *)[user.date objectForKey:@"day"]).intValue;
    components.timeZone = calendar.timeZone;
//    NSString *date = [formatter stringFromDate:[calendar dateFromComponents:components]];
    
//    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"RoomType"];
//    request.sortDescriptors = @[[NSSortDescriptor sortDescriptorWithKey:@"id" ascending:YES]];
//    NSManagedObjectContext *context = [Utils getContext];
//    NSError *error;
//    NSArray<RoomType *> *types = (NSArray<RoomType *> *)[context executeFetchRequest:request error:&error];
//    
//    int weight = 0;
//    for(RoomType *roomType in types) {
//        for(Room *room in roomType.rooms) {
//            for(RoomFurniture *roomFurniture in room.roomFurnitures) {
//                weight += roomFurniture.count.intValue * roomFurniture.furniture.weight.intValue;
//            }
//        }
//    }
    
    NSMutableDictionary *content = [[NSMutableDictionary alloc] init];
    
    [content setObject:user.name forKey:@"name"];
    [content setObject:user.email forKey:@"email"];
    [content setObject:user.phone forKey:@"phone"];
    
    [content setObject:[user.zip objectForKey:@"zip"] forKey:@"from_postal_code"];
    [content setObject:@"US" forKey:@"to_country"];
    [content setObject:[user.state objectForKey:@"abr"] forKey:@"to_admin1"];
    [content setObject:[user.city objectForKey:@"name"] forKey:@"to_city"];
    
    //[content setObject:date forKey:@"shipping_date"];
    [content setObject:[user.date objectForKey:@"year"] forKey:@"year"];
    [content setObject:[user.date objectForKey:@"month"] forKey:@"month"];
    [content setObject:[user.date objectForKey:@"day"] forKey:@"date"];
    
    [content setObject:[user.make objectForKey:@"make_name"] forKey:@"make"];
    [content setObject:[user.model objectForKey:@"model_name"] forKey:@"model"];
    [content setObject:[user.condition objectForKey:@"name"] forKey:@"vehicle_condition"];
    [content setObject:[user.method objectForKey:@"name"] forKey:@"shipping"];
    
//    [content setObject:[user.size objectForKey:@"quoteName"] forKey:@"movingSize"];
//    [content setObject:[NSNumber numberWithInt:weight] forKey:@"move_weight"];
//    [content setObject:@"MA-iOS-2.0" forKey:@"domain_from"];

    NSString *url = @"https://www.mymovingloads.com/vehicle-moving-quote-usa.htm";
    [URLRequests sendRequest:url type:RequestTypeGet params:content receive:^(NSData *data) {
        NSError *jsonerror;
        id obj = [NSJSONSerialization JSONObjectWithData:data options:0 error:&jsonerror];
        if(obj != nil && [obj isKindOfClass:[NSDictionary class]]) {
            NSString *status = (NSString *)[obj objectForKey:@"status"];
            BOOL success;
            NSMutableString *errors;
            if(![status isEqualToString:@"error"]) {
                success = YES;
            }
            else {
                success = NO;
                errors = [[NSMutableString alloc] init];
                for(NSString *error in (NSArray *)[obj objectForKey:@"errors"]) {
                    if(errors.length > 0) {
                        [errors appendString:@"\n"];
                    }
                    [errors appendString:error];
                }
            }
            if(receive != nil) {
                receive(success, errors);
            }
        }
    }];
}

+ (void)sendEmail:(NSDictionary *)data to:(NSString *)to receive:(void (^)(BOOL success, NSString *error))receive {
    NSMutableDictionary *params = [[NSMutableDictionary alloc] init];
    
    for(NSString *key in data) {
        [params setObject:[data objectForKey:key] forKey:key];
    }
    
    NSMutableDictionary *content = [[NSMutableDictionary alloc] init];
    
    UIDevice *device = [UIDevice currentDevice];
    
    [content setObject:device.name forKey:@"device"];
    [content setObject:device.model forKey:@"model"];
    [content setObject:device.name forKey:@"product"];
    [content setObject:@"apple" forKey:@"manufacturer"];
    [content setObject:@"apple" forKey:@"brand"];
    [content setObject:@"serial" forKey:@"serial"];
    [content setObject:@"fingerprint" forKey:@"fingerprint"];
    [content setObject:@"hardware" forKey:@"hardware"];
    [content setObject:@"cpu_abi" forKey:@"cpu_abi"];
    [content setObject:@"cpu_abi2" forKey:@"cpu_abi2"];
    [content setObject:@"tags" forKey:@"tags"];
    
    [content setObject:device.systemVersion forKey:@"api"];
    [content setObject:device.systemVersion forKey:@"release"];
    [content setObject:device.systemName forKey:@"codename"];

    [content setObject:@"name" forKey:@"name"];
    [content setObject:@"phone_number" forKey:@"phone_number"];
    [content setObject:@"email_address" forKey:@"email_address"];
    
    [params setObject:content forKey:@"data"];

    NSString *url = @"https://www.mymovingloads.com/services/emails/ma-";
    url = [url stringByAppendingString:to];
    [URLRequests sendRequest:url type:RequestTypePost params:params receive:^(NSData *data) {
        NSError *jsonerror;
        id obj = [NSJSONSerialization JSONObjectWithData:data options:0 error:&jsonerror];
        if(obj != nil && [obj isKindOfClass:[NSDictionary class]]) {
            NSString *status = (NSString *)[obj objectForKey:@"status"];
            BOOL success;
            NSMutableString *errors;
            if(![status isEqualToString:@"error"]) {
                success = YES;
            }
            else {
                success = NO;
                errors = [[NSMutableString alloc] init];
                for(NSString *error in (NSArray *)[obj objectForKey:@"errors"]) {
                    if(errors.length > 0) {
                        [errors appendString:@"\n"];
                    }
                    [errors appendString:error];
                }
            }
            if(receive != nil) {
                receive(success, errors);
            }
        }
    }];
}

+ (void)getTexts {
    NSString *url = @"https://www.moveadvisor.com/rideshipper/app_ios/vars.htm";
    [URLRequests sendRequest:url type:RequestTypeGet params:nil receive:^(NSData *data) {
        NSError *jsonerror;
        id obj = [NSJSONSerialization JSONObjectWithData:data options:0 error:&jsonerror];
        if(obj != nil && [obj isKindOfClass:[NSDictionary class]]) {
            NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Text"];
            NSManagedObjectContext *context = [Utils getContext];
            NSError *error;
            NSArray *current = [context executeFetchRequest:request error:&error];
            if(current != nil) {
                for(Text *text in current) {
                    [context deleteObject:text];
                }
                [Utils saveContext];
            }
            for(NSString *key in (NSDictionary *)obj) {
                Text *text = [NSEntityDescription insertNewObjectForEntityForName:@"Text" inManagedObjectContext:context];
                text.name = key;
                text.value = (NSString *)[(NSDictionary *)obj objectForKey:key];
            }
            [Utils saveContext];
        }
    }];
}

@end
